'''
大宗交易
https://www.okx.com/docs-v5/zh/#block-trading-rest-api
'''

from paux.param import to_local
from okx.api._client import Client


class _BlockTradeEndpoints():
    get_counterparties = ['/api/v5/rfq/counterparties', 'GET']  # 获取报价方信息
    set_create_rfq = ['/api/v5/rfq/create-rfq', 'POST']  # 询价
    set_cancel_rfq = ['/api/v5/rfq/cancel-rfq', 'POST']  # 取消询价单
    set_cancel_batch_rfqs = ['/api/v5/rfq/cancel-batch-rfqs', 'POST']  # 批量取消询价单
    set_cancel_all_rfqs = ['/api/v5/rfq/cancel-all-rfqs', 'POST']  # 取消所有询价单
    set_execute_quote = ['/api/v5/rfq/execute-quote', 'POST']  # 执行报价
    get_maker_instrument_settings = ['/api/v5/rfq/maker-instrument-settings', 'GET']  # 获取可报价产品
    set_maker_instrument_settings = ['/api/v5/rfq/maker-instrument-settings', 'POST']  # 设置可报价产品
    set_mmp_reset = ['/api/v5/rfq/mmp-reset', 'POST']  # 重设MMP状态
    set_create_quote = ['/api/v5/rfq/create-quote', 'POST']  # 报价
    set_cancel_quote = ['/api/v5/rfq/cancel-quote', 'POST']  # 取消报价单
    set_cancel_batch_quotes = ['/api/v5/rfq/cancel-batch-quotes', 'POST']  # 批量取消报价单
    set_cancel_all_quotes = ['/api/v5/rfq/cancel-all-quotes', 'POST']  # 取消所有报价单
    get_rfqs = ['/api/v5/rfq/rfqs', 'GET']  # 获取询价单信息
    get_quotes = ['/api/v5/rfq/quotes', 'GET']  # 获取报价单信息
    get_trades = ['/api/v5/rfq/trades', 'GET']  # 获取大宗交易信息
    get_public_trades = ['/api/v5/rfq/public-trades', 'GET']  # 获取大宗交易公共成交数据
    get_block_tickers = ['/api/v5/market/block-tickers', 'GET']  # 获取大宗交易所有产品行情信息
    get_block_ticker = ['/api/v5/market/block-ticker', 'GET']  # 获取大宗交易单个产品行情信息
    get_block_trades = ['/api/v5/market/block-trades', 'GET']  # 获取大宗交易公共成交数据


class BlockTrade(Client):

    # 获取报价方信息
    def get_counterparties(self, proxies={}, proxy_host: str = None):
        '''
        查询可以参与交易的报价方信息。
        https://www.okx.com/docs-v5/zh/#block-trading-rest-api-get-counterparties
        
        
        限速规则：UserID
    
        
        返回参数:
        Parameter         	Type    	Description
        traderName        	String  	报价方名称
        traderCode        	String  	报价方唯一标识代码，公开可见；报价和询价的相关接口都使用该代码代表报价方。
        type              	String  	报价方类型。LP指通过API连接的自动做市商。
        '''
        return self.send_request(*_BlockTradeEndpoints.get_counterparties, **to_local(locals()))

    # 询价
    def set_create_rfq(self, counterparties: object, legs: object, anonymous: bool = '', clRfqId: str = '',
                       tag: str = '', allowPartialExecution: bool = '', proxies={}, proxy_host: str = None):
        '''
        创建一个询价单。
        https://www.okx.com/docs-v5/zh/#block-trading-rest-api-create-rfq
        
        
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        counterparties    	Array of strings	是       	希望收到询价的报价方列表，可通过/api/v5/rfq/counterparties/获取。
        anonymous         	Boolean 	否       	是否匿名询价，true表示匿名询价，false表示公开询价，默认值为false，为true时，即使在交易执行之后，身份也不会透露给报价方。
        clRfqId           	String  	否       	询价单自定义ID，字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间。
        tag               	String  	否       	询价单标签，与此询价单关联的大宗交易将有相同的标签。字母（区分大小写）与数字的组合，可以是纯字母、纯数字，且长度在1-16位之间。
        allowPartialExecution	Boolean 	否       	RFQ是否可以被部分执行，如果腿的比例和原RFQ一致。有效值为true或false。默认为false。
        legs              	Array of objects	是       	组合交易，每次最多可以提交15组交易信息
        > instId          	String  	是       	产品ID
        > tdMode          	String  	否       	交易模式保证金模式：cross全仓isolated逐仓非保证金模式：cash非保证金.如未提供，tdMode 将继承系统设置的默认值：单币种保证金模式 & 现货:cash单币种保证金和多币种保证金模式下买入期权：isolated其他情况:cross
        > ccy             	String  	否       	保证金币种，仅适用于单币种保证金模式下的全仓杠杆订单在其他情况下该参数将被忽略。
        > sz              	String  	是       	委托数量
        > side            	String  	是       	询价单方向
        > posSide         	String  	否       	持仓方向买卖模式下默认为net。在开平仓模式下仅可选择long或short。如未指定，则处于开平仓模式下的用户始终会开新仓位。仅适用交割、永续。
        > tgtCcy          	String  	否       	委托数量的类型定义sz属性的单位。仅适用于 instType=SPOT。有效值为base_ccy和quote_ccy。未指定时，默认为base_ccy。
        返回参数:
        Parameter         	Type    	Description
        code              	String  	结果代码，0 表示成功。
        msg               	String  	错误信息，如果代码不为 0，则不为空。
        data              	Array of objects	询价单结果
        > cTime           	String  	询价单创建时间，Unix时间戳的毫秒数格式。
        > uTime           	String  	询价单状态更新时间，Unix时间戳的毫秒数格式。
        > state           	String  	询价单的状态有效值为activecanceledpending_fillfilledexpiredtraded_awayfailedtraded_away仅适用于报价方
        > counterparties  	Array of strings	报价方列表
        > validUntil      	String  	询价单的过期时间，Unix时间戳的毫秒数格式。
        > clRfqId         	String  	询价单自定义ID，为客户端敏感信息，不会公开，对报价方返回""。
        > tag             	String  	RFQ标签，与此RFQ关联的大宗交易将有相同的标签。
        > allowPartialExecution	Boolean 	RFQ是否可以被部分执行，如果腿的比例和原RFQ一致。有效值为true或false。未指定时，默认为false。
        > traderCode      	String  	询价方唯一标识代码。
        > rfqId           	String  	询价单ID
        > legs            	Array of objects	组合交易，每个请求最多可放置15条腿
        >> instId         	String  	产品ID，例如："BTC-USDT-SWAP"
        >> tdMode         	String  	交易模式保证金模式：cross全仓isolated逐仓非保证金模式：cash非保证金.如未提供，tdMode 将继承系统设置的默认值：单币种保证金模式 & 现货:cash单币种保证金和多币种保证金模式下买入期权：isolated其他情况:cross
        >> ccy            	String  	保证金币种，仅适用于单币种保证金模式下的全仓杠杆订单在其他情况下该参数将被忽略。
        >> sz             	String  	委托数量
        >> side           	String  	询价单方向有效值为buy和sell。
        >> posSide        	String  	持仓方向买卖模式下默认为net。如未指定，则返回""，相当于net。在开平仓模式下仅可选择long或short。 如未指定，则返回""，对应于为交易开新仓位的方向（买入=>long，卖出=>short）。仅适用交割、永续。
        >> tgtCcy         	String  	委托数量的类型定义sz属性的单位。仅适用于 instType=SPOT。有效值为base_ccy和quote_ccy。未指定时，默认为base_ccy。
        '''
        return self.send_request(*_BlockTradeEndpoints.set_create_rfq, **to_local(locals()))

    # 取消询价单
    def set_cancel_rfq(self, rfqId: str = '', clRfqId: str = '', proxies={}, proxy_host: str = None):
        '''
        取消一个询价单。
        https://www.okx.com/docs-v5/zh/#block-trading-rest-api-cancel-rfq
        
        
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        rfqId             	String  	可选      	询价单ID
        clRfqId           	String  	可选      	询价单自定义ID，字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间。当 clRfqId 和 rfqId 都传时，以 rfqId 为准。
        返回参数:
        Parameter         	Type    	Description
        code              	String  	结果代码，0 表示成功。
        msg               	String  	错误信息，如果代码不为 0，则不为空。
        data              	Array of objects	包含结果的对象数组
        > rfqId           	String  	RFQ ID
        > clRfqId         	String  	由用户设置的 RFQ ID
        > sCode           	String  	事件执行结果的code，0代表成功
        > sMsg            	String  	事件执行失败时的msg
        '''
        return self.send_request(*_BlockTradeEndpoints.set_cancel_rfq, **to_local(locals()))

    # 批量取消询价单
    def set_cancel_batch_rfqs(self, rfqIds: object = '', clRfqIds: object = '', proxies={}, proxy_host: str = None):
        '''
        取消一个或多个询价单，每次最多可以撤销100个询价单。
        https://www.okx.com/docs-v5/zh/#block-trading-rest-api-cancel-multiple-rfqs
        
        
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        rfqIds            	Array of strings	可选      	询价单IDs
        clRfqIds          	Array of strings	可选      	询价单自定义ID，当 clRfqIds 和 rfqIds 都传时，以 rfqIds 为准。
        返回参数:
        Parameter         	Type    	Description
        code              	String  	结果代码，0 表示成功。
        msg               	String  	错误信息，如果代码不为 0，则不为空。
        data              	Array of objects	包含结果的对象数组
        > rfqId           	String  	询价单ID
        > clRfqId         	String  	询价单自定义ID.
        > sCode           	String  	事件执行结果的code，0代表成功
        > sMsg            	String  	事件执行失败时的msg
        '''
        return self.send_request(*_BlockTradeEndpoints.set_cancel_batch_rfqs, **to_local(locals()))

    # 取消所有询价单
    def set_cancel_all_rfqs(self, proxies={}, proxy_host: str = None):
        '''
        取消所有询价单
        https://www.okx.com/docs-v5/zh/#block-trading-rest-api-cancel-all-rfqs
        
        
        限速规则：UserID
    
        
        返回参数:
        Parameter         	Type    	Description
        code              	String  	结果代码，0 表示成功。
        msg               	String  	错误信息，如果代码不为 0，则不为空。
        data              	Array of objects	包含结果的对象数组
        > ts              	String  	成功取消时间，Unix时间戳的毫秒数格式，例如 1597026383085。
        '''
        return self.send_request(*_BlockTradeEndpoints.set_cancel_all_rfqs, **to_local(locals()))

    # 执行报价
    def set_execute_quote(self, rfqId: str, quoteId: str, legs: object = '', proxies={}, proxy_host: str = None):
        '''
        执行报价，仅限询价的创建者使用
        https://www.okx.com/docs-v5/zh/#block-trading-rest-api-execute-quote
        
        
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        rfqId             	String  	是       	询价单ID
        quoteId           	String  	是       	报价单ID
        legs              	Array of objects	否       	用于部分执行的腿的数量。腿的数量比例必须与原RFQ相同。注意：每条腿的tgtCcy和side和原RFQ一致，px和对应Quote一致。
        > instId          	String  	是       	产品ID, 如 "BTC-USDT-SWAP".
        > sz              	String  	是       	该条腿的部分执行数量
        返回参数:
        Parameter         	Type    	Description
        code              	String  	结果代码，0 表示成功。
        msg               	String  	错误信息，如果代码不为 0，则不为空。
        data              	Array of objects	包含结果的对象数组
        > cTime           	String  	交易执行的时间，Unix时间戳的毫秒数格式。
        > rfqId           	String  	询价单ID
        > clRfqId         	String  	询价单自定义ID，为客户敏感信息，不会公开，对报价方返回""。
        > quoteId         	String  	报价单ID
        > clQuoteId       	String  	报价单自定义ID，为客户敏感信息，不会公开，对询价方返回""。
        > blockTdId       	String  	大宗交易ID
        > tag             	String  	报价单标签，与此报价单关联的大宗交易将有相同的标签。
        > tTraderCode     	String  	询价价方唯一标识代码。询价时 anonymous 设置为true时不可见。
        > mTraderCode     	String  	报价方唯一标识代码。 报价时 anonymous 设置为true时不可见。
        > legs            	Array of objects	组合交易
        >> instId         	String  	产品ID
        >> px             	String  	成交价格
        >> sz             	String  	成交数量
        >> side           	String  	询价单方向，buy或者sell。
        >> fee            	String  	手续费，正数代表平台返佣 ，负数代表平台扣除
        >> feeCcy         	String  	手续费币种
        >> tradeId        	String  	最新的成交Id.
        '''
        return self.send_request(*_BlockTradeEndpoints.set_execute_quote, **to_local(locals()))

    # 获取可报价产品
    def get_maker_instrument_settings(self, proxies={}, proxy_host: str = None):
        '''
        用于maker查询特定的接受询价和报价的产品, 以及数量和价格范围。
        https://www.okx.com/docs-v5/zh/#block-trading-rest-api-get-quote-products
        
        
        限速规则：UserID
    
        
        返回参数:
        Parameter         	Type    	Description
        code              	String  	结果代码，0表示成功
        msg               	String  	错误信息，如果代码不为0，则不为空
        data              	Array of objects	请求返回值，包含请求结果
        instType          	String  	产品类别，枚举值包括FUTURES,OPTION,SWAP和SPOT
        includeAll        	Boolean 	是否接收该instType下所有产品。有效值为true或false。默认false。
        > data            	Array of objects	instType的元素
        >> instFamily     	String  	交易品种交割/永续/期权情况下必填
        >> instId         	String  	产品ID，如BTC-USDT。对SPOT产品类别有效且必须。
        >> maxBlockSz     	String  	该种产品最大可交易数量。FUTURES, OPTION and SWAP 的单位是合约数量。SPOT的单位是交易货币。
        >> makerPxBand    	String  	价格限制以价格精度tick为单位，以标记价格为基准。设置makerPxBand为1个tick代表:如果买一价  > 标记价格 + 1 tick, 操作将被拦截如果 买一价 < 标记价格 - 1 tick, 操作将被拦截
        '''
        return self.send_request(*_BlockTradeEndpoints.get_maker_instrument_settings, **to_local(locals()))

    # 设置可报价产品
    def set_maker_instrument_settings(self, instType: str, data: object, includeAll: bool = '', proxies={},
                                      proxy_host: str = None):
        '''
        用于maker设置特定的接受询价和报价的产品, 以及数量和价格范围。
        https://www.okx.com/docs-v5/zh/#block-trading-rest-api-set-quote-products
        
        
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        instType          	String  	是       	产品类别，枚举值包括FUTURES,OPTION,SWAP和SPOT
        includeAll        	Boolean 	否       	是否接收该instType下所有产品。有效值为true或false。默认false。
        data              	Array of objects	是       	instType的元素
        > instFamily      	String  	可选      	交易品种交割/永续/期权情况下必填
        > instId          	String  	可选      	产品ID，如BTC-USDT。对SPOT产品类别有效且必须。
        > maxBlockSz      	String  	否       	该种产品最大可交易数量。FUTURES, OPTION and SWAP 的单位是合约数量。SPOT的单位是交易货币。
        > makerPxBand     	String  	否       	价格限制以价格精度tick为单位，以标记价格为基准。以设置makerPxBand为1个tick为例:如果买价  > 标记价格 + 1 tick, 操作将被拦截如果卖价 < 标记价格 - 1 tick, 操作将被拦截
        返回参数:
        Parameter         	Type    	Description
        code              	String  	结果代码，0表示成功
        msg               	String  	错误信息，如果代码不为0，则不为空
        data              	Array of objects	请求返回值，包含请求结果
        > result          	Boolean 	请求结果，枚举值为true,false
        '''
        return self.send_request(*_BlockTradeEndpoints.set_maker_instrument_settings, **to_local(locals()))

    # 重设MMP状态
    def set_mmp_reset(self, proxies={}, proxy_host: str = None):
        '''
        重设MMP状态为无效。
        https://www.okx.com/docs-v5/zh/#block-trading-rest-api-reset-mmp-status
        
        
        限速规则：UserID
    
        
        返回参数:
        Parameter         	Type    	Description
        code              	String  	结果代码，0表示成功
        msg               	String  	错误信息，如果代码不为0，则不为空
        data              	Array of objects	请求返回值，包含请求结果
        ts                	String  	重设时间. Unix 时间戳的毫秒数格式，如1597026383085.
        '''
        return self.send_request(*_BlockTradeEndpoints.set_mmp_reset, **to_local(locals()))

    # 报价
    def set_create_quote(self, rfqId: str, quoteSide: str, legs: object, clQuoteId: str = '', tag: str = '',
                         anonymous: bool = '', expiresIn: str = '', proxies={}, proxy_host: str = None):
        '''
        允许询价单指定的报价方进行报价，需要对整个询价单报价，不允许部分报价。
        https://www.okx.com/docs-v5/zh/#block-trading-rest-api-create-quote
        
        
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        rfqId             	String  	是       	询价单ID
        clQuoteId         	String  	否       	报价单自定义ID
        tag               	String  	否       	报价单标签，与此报价单关联的大宗交易将有相同的标签。字母（区分大小写）与数字的组合，可以是纯字母、纯数字，且长度在1-16位之间。
        anonymous         	Boolean 	否       	是否匿名报价，true表示匿名报价，false表示公开报价，默认值为false，为true时，即使在交易执行之后，身份也不会透露给询价方。
        quoteSide         	String  	是       	报价单方向，buy或者sell。当报价单方向为buy，对maker来说，执行方向与legs里的方向相同，对taker来说相反。反之同理
        expiresIn         	String  	否       	报价单的有效时长（以秒为单位）。 10到120之间的任何整数。 默认值为60
        legs              	Array of objects	是       	组合交易
        > instId          	String  	是       	产品ID
        > tdMode          	String  	否       	交易模式保证金模式：cross全仓isolated逐仓非保证金模式：cash非保证金.如未提供，tdMode 将继承系统设置的默认值：单币种保证金模式 & 现货:cash单币种保证金和多币种保证金模式下买入期权：isolated其他情况:cross
        > ccy             	String  	否       	保证金币种，仅适用于单币种保证金模式下的全仓杠杆订单在其他情况下该参数将被忽略。
        > sz              	String  	是       	委托数量
        > px              	String  	是       	委托价格
        > side            	String  	是       	报价单方向
        > posSide         	String  	否       	持仓方向买卖模式下默认为net。在开平仓模式下仅可选择long或short。如未指定，则处于开平仓模式下的用户始终会开新仓位。仅适用交割、永续。
        > tgtCcy          	String  	否       	委托数量的类型定义sz属性的单位。仅适用于 instType=SPOT。有效值为base_ccy和quote_ccy。未指定时，默认为base_ccy。
        返回参数:
        Parameter         	Type    	Description
        code              	String  	结果代码，0表示成功。
        msg               	String  	错误信息，如果代码不为0，则不为空。
        data              	Array of objects	包含结果的对象数组
        > cTime           	String  	报价单创建时间，Unix时间戳的毫秒数格式。
        > uTime           	String  	报价单状态更新时间，Unix时间戳的毫秒数格式。
        > state           	String  	报价单的状态有效值为activecanceledpending_fillfilledexpiredfailed
        > reason          	String  	状态原因. 有效值包括mmp_canceled.
        > validUntil      	String  	报价单的过期时间，Unix时间戳的毫秒数格式。
        > rfqId           	String  	询价单ID
        > clRfqId         	String  	询价单自定义ID，为客户敏感信息，不会公开，对报价方返回""。
        > quoteId         	String  	报价单ID
        > clQuoteId       	String  	报价单自定义ID，为客户敏感信息，不会公开，对询价方返回""。
        > tag             	String  	报价单标签，与此报价单关联的大宗交易将有相同的标签。
        > traderCode      	String  	报价方唯一标识代码。
        > quoteSide       	String  	报价单方向，有效值为buy或者sell。当报价单方向为buy，对maker来说，执行方向与legs里的方向相同，对taker来说相反。反之同理。
        > legs            	Array of objects	组合交易
        >> instId         	String  	产品ID
        >> tdMode         	String  	交易模式保证金模式：cross全仓isolated逐仓非保证金模式：cash非保证金.如未提供，tdMode 将继承系统设置的默认值：单币种保证金模式 & 现货:cash单币种保证金和多币种保证金模式下买入期权：isolated其他情况:cross
        >> ccy            	String  	保证金币种，仅适用于单币种保证金模式下的全仓杠杆订单在其他情况下该参数将被忽略。
        >> sz             	String  	委托数量
        >> px             	String  	委托价格
        >> side           	String  	腿的方向，有效值为buy或者sell。
        >> posSide        	String  	持仓方向买卖模式下默认为net。如未指定，则返回""，相当于net。在开平仓模式下仅可选择long或short。 如未指定，则返回""，对应于为交易开新仓位的方向（买入=>long，卖出=>short）。仅适用交割、永续。
        >> tgtCcy         	String  	委托数量的类型定义sz属性的单位。仅适用于 instType=SPOT。有效值为base_ccy和quote_ccy。未指定时，默认为base_ccy。
        '''
        return self.send_request(*_BlockTradeEndpoints.set_create_quote, **to_local(locals()))

    # 取消报价单
    def set_cancel_quote(self, quoteId: str = '', clQuoteId: str = '', rfqId: str = '', proxies={},
                         proxy_host: str = None):
        '''
        取消一个报价单。
        https://www.okx.com/docs-v5/zh/#block-trading-rest-api-cancel-quote
        
        
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        quoteId           	String  	可选      	报价单ID
        clQuoteId         	String  	可选      	报价单自定义ID，字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间，当 clRfqId 和 rfqId 都传时，以 rfqId 为准。
        rfqId             	String  	否       	询价单ID
        返回参数:
        Parameter         	Type    	Description
        code              	String  	结果代码，0 表示成功。
        msg               	String  	错误信息，如果代码不为 0，则不为空。
        data              	Array of objects	包含结果的对象数组
        > quoteId         	String  	报价单ID
        > clQuoteId       	String  	询价单自定义ID
        > sCode           	String  	事件执行结果的code，0代表成功
        > sMsg            	String  	事件执行失败时的msg
        '''
        return self.send_request(*_BlockTradeEndpoints.set_cancel_quote, **to_local(locals()))

    # 批量取消报价单
    def set_cancel_batch_quotes(self, quoteIds: object = '', clQuoteIds: object = '', proxies={},
                                proxy_host: str = None):
        '''
        取消一个或多个报价单，每次最多可以撤销100个订单。
        https://www.okx.com/docs-v5/zh/#block-trading-rest-api-cancel-multiple-quotes
        
        
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        quoteIds          	Array of strings	可选      	报价单ID
        clQuoteIds        	Array of strings	可选      	报价单自定义ID，当 clQuoteIds 和 quoteIds 都传时，以 quoteIds 为准。
        返回参数:
        Parameter         	Type    	Description
        code              	String  	结果代码，0 表示成功。
        msg               	String  	错误信息，如果代码不为 0，则不为空。
        data              	Array of objects	包含结果的对象数组
        > quoteId         	String  	报价单ID
        > clQuoteId       	String  	报价单自定义ID
        > sCode           	String  	事件执行结果的code，0代表成功
        > sMsg            	String  	事件执行失败时的msg
        '''
        return self.send_request(*_BlockTradeEndpoints.set_cancel_batch_quotes, **to_local(locals()))

    # 取消所有报价单
    def set_cancel_all_quotes(self, proxies={}, proxy_host: str = None):
        '''
        取消所有报价单
        https://www.okx.com/docs-v5/zh/#block-trading-rest-api-cancel-all-quotes
        
        
        限速规则：UserID
    
        
        返回参数:
        Parameter         	Type    	Description
        code              	String  	结果代码，0 表示成功。
        msg               	String  	错误信息，如果代码不为 0，则不为空。
        data              	Array of objects	包含结果的对象数组
        > ts              	String  	成功取消时间，Unix时间戳的毫秒数格式，例如 1597026383085。
        '''
        return self.send_request(*_BlockTradeEndpoints.set_cancel_all_quotes, **to_local(locals()))

    # 获取询价单信息
    def get_rfqs(self, rfqId: str = '', clRfqId: str = '', state: str = '', beginId: str = '', endId: str = '',
                 limit: str = '', proxies={}, proxy_host: str = None):
        '''
        获取用户发出的或收到的询价单信息
        https://www.okx.com/docs-v5/zh/#block-trading-rest-api-get-rfqs
        
        
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        rfqId             	String  	否       	询价单ID .
        clRfqId           	String  	否       	客户询价单自定义ID，当 clRfqId 和 rfqId 都传时，以 rfqId 为准
        state             	String  	否       	询价单的状态activecanceledpending_fillfilledexpiredfailedtraded_awaytraded_away仅适用于报价方
        beginId           	String  	否       	请求的起始询价单ID，请求此ID之后（更新的数据）的分页内容，不包括 beginId
        endId             	String  	否       	请求的结束询价单ID，请求此ID之前（更旧的数据）的分页内容，不包括 endId
        limit             	String  	否       	返回结果的数量，最大为100，默认100条
        返回参数:
        Parameter         	Type    	Description
        code              	String  	结果代码，0 表示成功。
        msg               	String  	错误信息，如果代码不为 0，则不为空。
        data              	Array of objects	包含结果的对象数组
        > cTime           	String  	询价单创建时间，Unix时间戳的毫秒数格式。
        > uTime           	String  	询价单状态更新时间，Unix时间戳的毫秒数格式。
        > state           	String  	询价单的状态activecanceledpending_fillfilledexpiredfailedtraded_awaytraded_away仅适用于报价方
        > counterparties  	Array of srings	报价方列表
        > validUntil      	String  	询价单的过期时间，Unix时间戳的毫秒数格式。
        > clRfqId         	String  	询价单自定义ID，为客户敏感信息，不会公开，对报价方返回""。
        > tag             	String  	询价单标签，与此询价单关联的大宗交易将有相同的标签。
        > traderCode      	String  	询价方唯一标识代码，询价时 anonymous 设置为true时不可见
        > rfqId           	String  	询价单ID
        > allowPartialExecution	Boolean 	RFQ是否可以被部分执行，如果腿的比例和原RFQ一致。有效值为true或false。未指定时，默认为false。
        > legs            	Array of objects	组合交易，每个请求最多可放置15条腿
        >> instId         	String  	产品ID，例如："BTC-USDT-SWAP"
        >> tdMode         	String  	交易模式保证金模式：cross全仓isolated逐仓非保证金模式：cash非保证金.如未提供，tdMode 将继承系统设置的默认值：单币种保证金模式 & 现货:cash单币种保证金和多币种保证金模式下买入期权：isolated其他情况:cross
        >> ccy            	String  	保证金币种，仅适用于单币种保证金模式下的全仓杠杆订单在其他情况下该参数将被忽略。
        >> sz             	String  	委托数量
        >> side           	String  	询价单方向有效值为buy和sell。
        >> posSide        	String  	持仓方向买卖模式下默认为net。如未指定，则返回""，相当于net。在开平仓模式下仅可选择long或short。 如未指定，则返回""，对应于为交易开新仓位的方向（买入=>long，卖出=>short）。仅适用交割、永续。
        >> tgtCcy         	String  	委托数量的类型定义sz属性的单位。仅适用于 instType=SPOT。有效值为base_ccy和quote_ccy。未指定时，默认为base_ccy。
        '''
        return self.send_request(*_BlockTradeEndpoints.get_rfqs, **to_local(locals()))

    # 获取报价单信息
    def get_quotes(self, rfqId: str = '', clRfqId: str = '', quoteId: str = '', clQuoteId: str = '', state: str = '',
                   beginId: str = '', endId: str = '', limit: str = '', proxies={}, proxy_host: str = None):
        '''
        获取用户发出的或收到的报价单信息
        https://www.okx.com/docs-v5/zh/#block-trading-rest-api-get-quotes
        
        
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        rfqId             	String  	否       	询价单ID
        clRfqId           	String  	否       	询价单自定义ID， 当 clRfqId 和 rfqId 都传时，以 rfqId 为准。
        quoteId           	String  	否       	报价单ID
        clQuoteId         	String  	否       	报价单自定义ID，当 clRfqId 和 rfqId 都传时，以 rfqId 为准。
        state             	String  	否       	报价单的状态有效值为activecanceledpending_fillfilledexpiredfailed
        beginId           	String  	否       	请求的起始报价单ID，请求此ID之后（更新的数据）的分页内容，不包括 beginId
        endId             	String  	否       	请求的结束报价单ID，请求此ID之前（更旧的数据）的分页内容，不包括 endId
        limit             	String  	否       	返回结果的数量，最大为100，默认100条
        返回参数:
        Parameter         	Type    	Description
        code              	String  	结果代码，0 表示成功。
        msg               	String  	错误信息，如果代码不为 0，则不为空。
        data              	Array of objects	包含结果的数组
        > cTime           	String  	报价单创建时间，Unix时间戳的毫秒数格式
        > uTime           	String  	报价单状态更新时间，Unix时间戳的毫秒数格式。
        > state           	String  	报价单的状态activecanceledpending_fillfilledexpiredfailed
        > reason          	String  	状态原因. 有效值包括mmp_canceled.
        > validUntil      	String  	报价单的过期时间，Unix时间戳的毫秒数格式。
        > rfqId           	String  	询价单ID
        > clRfqId         	String  	询价单自定义ID，为客户敏感信息，不会公开，对报价方返回""。
        > quoteId         	String  	报价单ID
        > clQuoteId       	String  	报价单自定义ID，为客户敏感信息，不会公开，对询价方返回""。
        > tag             	String  	报价单标签，与此报价单关联的大宗交易将有相同的标签。
        > traderCode      	String  	报价方唯一标识代码，报价时 Anonymous 设置为True时不可见。
        > quoteSide       	String  	报价单方向，buy或者sell。当报价单方向为buy，对maker来说，执行方向与legs里的方向相同，对taker来说相反。反之同理
        > legs            	Array of objects	组合交易
        >> instId         	String  	产品ID
        >> tdMode         	String  	交易模式保证金模式：cross全仓isolated逐仓非保证金模式：cash非保证金.如未提供，tdMode 将继承系统设置的默认值：单币种保证金模式 & 现货:cash单币种保证金和多币种保证金模式下买入期权：isolated其他情况:cross
        >> ccy            	String  	保证金币种，仅适用于单币种保证金模式下的全仓杠杆订单在其他情况下该参数将被忽略。
        >> sz             	String  	委托数量
        >> px             	String  	委托价格.
        >> side           	String  	报价单方向
        >> posSide        	String  	持仓方向买卖模式下默认为net。如未指定，则返回""，相当于net。在开平仓模式下仅可选择long或short。 如未指定，则返回""，对应于为交易开新仓位的方向（买入=>long，卖出=>short）。仅适用交割、永续。
        >> tgtCcy         	String  	委托数量的类型定义sz属性的单位。仅适用于 instType=SPOT。有效值为base_ccy和quote_ccy。未指定时，默认为base_ccy。
        '''
        return self.send_request(*_BlockTradeEndpoints.get_quotes, **to_local(locals()))

    # 获取大宗交易信息
    def get_trades(self, rfqId: str = '', clRfqId: str = '', quoteId: str = '', blockTdId: str = '',
                   clQuoteId: str = '', state: str = '', beginId: str = '', endId: str = '', beginTs: str = '',
                   endTs: str = '', limit: str = '', proxies={}, proxy_host: str = None):
        '''
        获取该用户大宗交易成交信息
        https://www.okx.com/docs-v5/zh/#block-trading-rest-api-get-trades
        
        
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        rfqId             	String  	否       	询价单ID
        clRfqId           	String  	否       	由用户设置的询价单ID. 如果clRfqId和rfqId都通过了，rfqId 将被视为主要
        quoteId           	String  	否       	报价单ID
        blockTdId         	String  	否       	大宗交易ID
        clQuoteId         	String  	否       	由用户设置的报价单ID。如果同时传递了clQuoteId和quoteId，则 quoteId 将被视为主要标识符
        state             	String  	否       	询价单的状态activecanceledpending_fillfilledexpiredfailedtraded_awaytraded_away仅适用于报价方
        beginId           	String  	否       	请求的起始大宗交易ID，请求此ID之后（更新的数据）的分页内容，不包括 beginId
        endId             	String  	否       	请求的结束大宗交易ID，请求此ID之前（更旧的数据）的分页内容，不包括 endId
        beginTs           	String  	否       	用开始时间戳筛选交易执行时间（UTC时区）。Unix时间戳的毫秒数格式，例如 1597026383085。
        endTs             	String  	否       	用结束时间戳筛选交易执行时间（UTC时区）。Unix时间戳的毫秒数格式，例如 1597026383085。
        limit             	String  	否       	返回结果的数量，最大为100，默认100条。如果请求范围内的交易数量大于100，则返回该范围内最近的100笔交易。
        返回参数:
        Parameter         	Type    	Description
        code              	String  	结果代码，0 表示成功。
        msg               	String  	错误信息，如果代码不为 0，则不为空。
        data              	Array of objects	包含结果的对象数组
        > cTime           	String  	执行创建的时间，Unix时间戳的毫秒数格式。
        > rfqId           	String  	询价单ID
        > clRfqId         	String  	询价单自定义ID，为客户敏感信息，不会公开，对报价方返回""。
        > quoteId         	String  	报价单ID
        > clQuoteId       	String  	报价单自定义ID，为客户敏感信息，不会公开，对询价方返回""。
        > blockTdId       	String  	大宗交易ID
        > tag             	String  	交易标签，大宗交易将有与其对应的询价单或报价单相同的标签。
        > tTraderCode     	String  	询价方唯一标识代码，询价时 anonymous 设置为true时不可见
        > mTraderCode     	String  	报价方唯一标识代码。报价时 anonymous 设置为true时不可见
        > legs            	Array of objects	组合交易
        >> instId         	String  	产品ID
        >> px             	String  	成交价格
        >> sz             	String  	成交数量
        >> side           	String  	询价单方向，buy 或者 sell。
        >> fee            	String  	手续费，正数代表平台返佣 ，负数代表平台扣除
        >> feeCcy         	String  	手续费币种
        >> tradeId        	String  	最新的成交Id
        '''
        return self.send_request(*_BlockTradeEndpoints.get_trades, **to_local(locals()))

    # 获取大宗交易公共成交数据
    def get_public_trades(self, beginId: str = '', endId: str = '', limit: str = '', proxies={},
                          proxy_host: str = None):
        '''
        获取最近执行的大宗交易。
        https://www.okx.com/docs-v5/zh/#block-trading-rest-api-get-public-trades
        
        
        限速规则：IP
    
        请求参数:
        Parameter         	Type    	Required	Description

        beginId           	String  	否       	请求的起始大宗交易ID，请求此ID之后（更新的数据）的分页内容，不包括 beginId
        endId             	String  	否       	请求的结束大宗交易ID，请求此ID之前（更旧的数据）的分页内容，不包括 endId
        limit             	String  	否       	返回结果的数量，最大为100，默认100条
        返回参数:
        Parameter         	Type    	Description
        code              	String  	结果代码，0 表示成功。
        msg               	String  	错误信息，如果代码不为 0，则不为空。
        data              	Array of objects	包含结果的对象数组.
        > cTime           	String  	执行创建的时间，Unix时间戳的毫秒数格式。
        > blockTdId       	String  	大宗交易ID
        > legs            	Array of objects	组合交易
        >> instId         	String  	产品ID
        >> px             	String  	成交价格
        >> sz             	String  	成交数量
        >> side           	String  	询价单方向，从 Taker的视角看
        >> tradeId        	String  	最新成交ID
        '''
        return self.send_request(*_BlockTradeEndpoints.get_public_trades, **to_local(locals()))

    # 获取大宗交易所有产品行情信息
    def get_block_tickers(self, instType: str, uly: str = '', instFamily: str = '', proxies={}, proxy_host: str = None):
        '''
        获取最近24小时大宗交易量
        https://www.okx.com/docs-v5/zh/#block-trading-rest-api-get-block-tickers
        
        限速：20次/2s
        限速规则：IP
    
        请求参数:
        Parameter         	Type    	Required	Description

        instType          	String  	是       	产品类型SPOT：币币SWAP：永续合约FUTURES：交割合约OPTION：期权
        uly               	String  	否       	标的指数适用于交割/永续/期权，如BTC-USD
        instFamily        	String  	否       	交易品种适用于交割/永续/期权，如BTC-USD
        返回参数:
        Parameter         	Type    	Description
        instId            	String  	产品ID
        instType          	String  	产品类型
        volCcy24h         	String  	24小时成交量，以币为单位如果是衍生品合约，数值为交易货币的数量。如果是币币/币币杠杆，数值为计价货币的数量。
        vol24h            	String  	24小时成交量，以张为单位如果是衍生品合约，数值为合约的张数。如果是币币/币币杠杆，数值为交易货币的数量。
        ts                	String  	数据产生时间，Unix时间戳的毫秒数格式，如1597026383085
        '''
        return self.send_request(*_BlockTradeEndpoints.get_block_tickers, **to_local(locals()))

    # 获取大宗交易单个产品行情信息
    def get_block_ticker(self, instId: str, proxies={}, proxy_host: str = None):
        '''
        获取最近24小时大宗交易量
        https://www.okx.com/docs-v5/zh/#central-limit-orderbook-market-data-get-block-ticker
        
        限速：20次/2s
        限速规则：IP
    
        请求参数:
        Parameter         	Type    	Required	Description

        instId            	String  	是       	产品ID，如BTC-USD-SWAP
        返回参数:
        Parameter         	Type    	Description
        instId            	String  	产品ID
        instType          	String  	产品类型
        volCcy24h         	String  	24小时成交量，以币为单位如果是衍生品合约，数值为交易货币的数量。如果是币币/币币杠杆，数值为计价货币的数量。
        vol24h            	String  	24小时成交量，以张为单位如果是衍生品合约，数值为合约的张数。如果是币币/币币杠杆，数值为交易货币的数量。
        ts                	String  	数据产生时间，Unix时间戳的毫秒数格式，如1597026383085
        '''
        return self.send_request(*_BlockTradeEndpoints.get_block_ticker, **to_local(locals()))

    # 获取大宗交易公共成交数据
    def get_block_trades(self, instId: str, proxies={}, proxy_host: str = None):
        '''
        查询市场上的成交信息数据，根据 tradeId 倒序排序。
        https://www.okx.com/docs-v5/zh/#central-limit-orderbook-market-data-get-block-trades
        
        限速：20次/2s
        限速规则：IP
    
        请求参数:
        Parameter         	Type    	Required	Description

        instId            	String  	是       	产品ID，如BTC-USDT
        返回参数:
        Parameter         	Type    	Description
        instId            	String  	产品ID
        tradeId           	String  	成交ID
        px                	String  	成交价格
        sz                	String  	成交数量
        side              	String  	成交方向buy：买sell：卖
        ts                	String  	成交时间，Unix时间戳的毫秒数格式， 如1597026383085
        '''
        return self.send_request(*_BlockTradeEndpoints.get_block_trades, **to_local(locals()))



from okx.app.market.exchange_info import ExchangeInfo
from okx.app.market.ticker import Ticker
from okx.app.market.history_candle import HistoryCandle


class Market(ExchangeInfo, Ticker, HistoryCandle):
    pass


class MarketSPOT(Market):
    def __init__(
            self,
            key: str = '',
            secret: str = '',
            passphrase: str = '',
            timezone='Asia/Shanghai',
            proxies={},
            proxy_host: str = None,
    ):
        instType = 'SPOT'
        super(MarketSPOT, self).__init__(
            instType=instType,
            key=key,
            secret=secret,
            passphrase=passphrase,
            timezone=timezone,
            proxies=proxies,
            proxy_host=proxy_host,

        )


class MarketSWAP(Market):
    def __init__(
            self,
            key: str = '',
            secret: str = '',
            passphrase: str = '',
            timezone='Asia/Shanghai',
            proxies={},
            proxy_host: str = None,
    ):
        instType = 'SWAP'
        super(MarketSWAP, self).__init__(
            instType=instType,
            key=key,
            secret=secret,
            passphrase=passphrase,
            timezone=timezone,
            proxies=proxies,
            proxy_host=proxy_host,
        )



from typing import Union
from copy import deepcopy
import numpy as np
import pandas as pd
from candlelite.crypto.okx_lite import OKX_TIMEZONE
from candlelite.calculate.transform import to_candle as _to_candle
from paux import date as _date


# 将Okx的历史K线数据从array转换成DataFrame类型
def candle_to_df(
        candle: np.ndarray,
        convert_ts: bool = True,
        timezone: Union[str, None] = OKX_TIMEZONE,
) -> pd.DataFrame:
    '''
    :param candle: array类型的历史K线数据
    :param convert_ts: 是否将时间戳转化为日期字符串
    :param timezone: 时区
        默认使用candlelite中Binance的时区
        None 使用本机默认时区
    '''
    df = pd.DataFrame(candle)
    df.columns = [
        'ts',  # 开盘时间 Open time
        'o',  # 开盘价 Open
        'h',  # 最高价 High
        'l',  # 最低价 Low
        'c',  # 收盘价 close
        'vol',  # 交易量，以张为单位；如果是衍生品合约，数值为合约的张数。如果是币币/币币杠杆，数值为交易货币的数量。
        'volCcy',  # 交易量，以币为单位；如果是衍生品合约，数值为交易货币的数量。如果是币币 / 币币杠杆，数值为计价货币的数量。
        'volCcyQuote',  # 交易量，以计价货币为单位；如：BTC-USDT 和 BTC-USDT-SWAP, 单位均是 USDT；BTC-USD-SWAP 单位是 USD
        'confirm',  # K线状态；0代表K线未完结，1代表K线已完结。
    ]
    # 是否转换时间戳为日期字符串
    if convert_ts:
        # 美国时间
        if timezone == 'America/New_York':
            fmt = '%m/%d/%Y %H:%M:%S'
        # 中国时间
        else:
            fmt = '%Y-%m-%d %H:%M:%S'
        df['ts'] = df['ts'].apply(
            lambda ts: _date.to_fmt(date=ts, timezone=timezone, fmt=fmt)
        )
    df['confirm'] = df['confirm'].astype(int)
    return df


# 将Okx的历史K线数据从DataFrame转换成array类型
def df_to_candle(
        df: pd.DataFrame,
        convert_ts: bool = True,
        timezone: Union[str, None] = OKX_TIMEZONE
):
    '''
    :param df: DataFrame类型的历史K线数据
    :param timezone: 时区
        默认使用candlelite中Binance的时区
        None 使用本机默认时区
    '''
    df = deepcopy(df)
    if convert_ts:
        df['ts'] = df['ts'].apply(
            lambda openTs: _date.to_ts(date=openTs, timezone=timezone)
        )
    candle = _to_candle(df)
    return candle



'''
跟单
https://www.okx.com/docs-v5/zh/#order-book-trading-copy-trading
'''
from paux.param import to_local
from okx.api._client import Client


class _CopyTradeEndpoints():
    get_current_subpositions = ['/api/v5/copytrading/current-subpositions', 'GET']  # GET / 交易员获取当前带单
    get_subpositions_history = ['/api/v5/copytrading/subpositions-history', 'GET']  # GET / 交易员获取历史带单
    set_algo_order = ['/api/v5/copytrading/algo-order', 'POST']  # POST / 交易员止盈止损
    set_close_subposition = ['/api/v5/copytrading/close-subposition', 'POST']  # POST / 交易员平仓
    get_instruments = ['/api/v5/copytrading/instruments', 'GET']  # GET / 交易员获取带单合约
    set_instruments = ['/api/v5/copytrading/set-instruments', 'POST']  # POST / 交易员修改带单合约
    get_profit_sharing_details = ['/api/v5/copytrading/profit-sharing-details', 'GET']  # GET / 交易员历史分润明细
    get_total_profit_sharing = ['/api/v5/copytrading/total-profit-sharing', 'GET']  # GET / 交易员历史分润汇总
    get_unrealized_profit_sharing_details = ['/api/v5/copytrading/unrealized-profit-sharing-details', 'GET']  # GET / 交易员待分润明细


class CopyTrade(Client):

    # GET / 交易员获取当前带单
    def get_current_subpositions(self, instId: str = '', after: str = '', before: str = '', limit: str = '', proxies={},
                                 proxy_host: str = None):
        '''
        交易员获取当前未平仓的带单仓位。
        https://www.okx.com/docs-v5/zh/#order-book-trading-copy-trading-get-existing-leading-positions
        
        限速：20次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        instId            	String  	否       	产品ID ，如BTC-USDT-SWAP
        after             	String  	否       	请求此id之前（更旧的数据）的分页内容，传的值为对应接口的subPosId
        before            	String  	否       	请求此id之后（更新的数据）的分页内容，传的值为对应接口的subPosId
        limit             	String  	否       	分页返回的结果集数量，最大为500，不填默认返回500条
        返回参数:
        Parameter         	Type    	Description
        instId            	String  	产品ID
        subPosId          	String  	带单仓位ID
        posSide           	String  	持仓方向long：开平仓模式开多short：开平仓模式开空net：买卖模式（subPos为正代表开多，subPos为负代表开空）
        mgnMode           	String  	保证金模式，isolated：逐仓 ；cross：全仓
        lever             	String  	杠杆倍数
        openOrdId         	String  	交易员开仓订单号
        openAvgPx         	String  	开仓均价
        openTime          	String  	开仓时间
        subPos            	String  	持仓张数
        tpTriggerPx       	String  	止盈触发价，触发后以市价进行委托
        slTriggerPx       	String  	止损触发价，触发后以市价进行委托
        algoId            	String  	止盈止损委托单ID
        '''
        return self.send_request(*_CopyTradeEndpoints.get_current_subpositions, **to_local(locals()))

    # GET / 交易员获取历史带单
    def get_subpositions_history(self, instId: str = '', after: str = '', before: str = '', limit: str = '', proxies={},
                                 proxy_host: str = None):
        '''
        交易员获取最近三个月的已经平仓的带单仓位，按照subPosId倒序排序。
        https://www.okx.com/docs-v5/zh/#order-book-trading-copy-trading-get-leading-position-history
        
        限速：20次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        instId            	String  	否       	产品ID ，如BTC-USDT-SWAP
        after             	String  	否       	请求此id之前（更旧的数据）的分页内容，传的值为对应接口的subPosId
        before            	String  	否       	请求此id之后（更新的数据）的分页内容，传的值为对应接口的subPosId
        limit             	String  	否       	分页返回的结果集数量，最大为100，不填默认返回100条
        返回参数:
        Parameter         	Type    	Description
        instId            	String  	产品ID
        subPosId          	String  	带单仓位ID
        posSide           	String  	持仓方向long：开平仓模式开多short：开平仓模式开空net：买卖模式（subPos为正代表开多，subPos为负代表开空）
        mgnMode           	String  	保证金模式，isolated：逐仓 ；cross：全仓
        lever             	String  	杠杆倍数
        openOrdId         	String  	交易员开仓订单号
        openAvgPx         	String  	开仓均价
        openTime          	String  	开仓时间
        subPos            	String  	持仓张数
        closeTime         	String  	平仓时间(最近一次平仓的时间，即完全平仓的时间)
        closeAvgPx        	String  	平仓均价
        pnl               	String  	收益额
        pnlRatio          	String  	收益率
        '''
        return self.send_request(*_CopyTradeEndpoints.get_subpositions_history, **to_local(locals()))

    # POST / 交易员止盈止损
    def set_algo_order(self, subPosId: str, tpTriggerPx: str = '', slTriggerPx: str = '', tpTriggerPxType: str = '',
                       slTriggerPxType: str = '', tag: str = '', proxies={}, proxy_host: str = None):
        '''
        交易员为当前未平仓的带单仓位设置止盈止损。
        https://www.okx.com/docs-v5/zh/#order-book-trading-copy-trading-post-place-leading-stop-order
        
        限速：20次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        subPosId          	String  	是       	带单仓位ID
        tpTriggerPx       	String  	可选      	止盈触发价，触发后以市价进行委托，tpTriggerPx 和 slTriggerPx 至少需要填写一个
        slTriggerPx       	String  	可选      	止损触发价，触发后以市价进行委托
        tpTriggerPxType   	String  	否       	止盈触发价类型last：最新价格index：指数价格mark：标记价格默认为last
        slTriggerPxType   	String  	否       	止损触发价类型last：最新价格index：指数价格mark：标记价格默认为last
        tag               	String  	否       	订单标签字母（区分大小写）与数字的组合，可以是纯字母、纯数字，且长度在1-16位之间。
        返回参数:
        Parameter         	Type    	Description
        subPosId          	String  	带单仓位ID
        tag               	String  	订单标签
        '''
        return self.send_request(*_CopyTradeEndpoints.set_algo_order, **to_local(locals()))

    # POST / 交易员平仓
    def set_close_subposition(self, subPosId: str, tag: str = '', proxies={}, proxy_host: str = None):
        '''
        交易员一次仅可平仓一个带单仓位。<!- 1-2-3 -->带单ID（subPosId）为必填参数，需要通过交易员获取当前带单接口获取。
        https://www.okx.com/docs-v5/zh/#order-book-trading-copy-trading-post-close-leading-position
        
        限速：20次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        subPosId          	String  	是       	带单仓位ID
        tag               	String  	否       	订单标签字母（区分大小写）与数字的组合，可以是纯字母、纯数字，且长度在1-16位之间。
        返回参数:
        Parameter         	Type    	Description
        subPosId          	String  	带单仓位ID
        tag               	String  	订单标签
        '''
        return self.send_request(*_CopyTradeEndpoints.set_close_subposition, **to_local(locals()))

    # GET / 交易员获取带单合约
    def get_instruments(self, proxies={}, proxy_host: str = None):
        '''
        交易员获取平台支持带单的合约。
        https://www.okx.com/docs-v5/zh/#order-book-trading-copy-trading-get-leading-instruments
        
        限速：5次/2s
        限速规则：UserID
    
        
        返回参数:
        Parameter         	Type    	Description
        instId            	String  	产品ID
        enabled           	Boolean 	是否设置了跟单true或false
        '''
        return self.send_request(*_CopyTradeEndpoints.get_instruments, **to_local(locals()))

    # POST / 交易员修改带单合约
    def set_instruments(self, instId: str, proxies={}, proxy_host: str = None):
        '''
        交易员修改带单合约的设置。初始带单合约在申请带单交易员时进行设置。<!- 1-2-3 -->非带单合约修改为带单合约时，该次请求中所有的非带单合约合约不能有持仓或者挂单。
        https://www.okx.com/docs-v5/zh/#order-book-trading-copy-trading-post-amend-leading-instruments
        
        限速：5次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        instId            	String  	是       	产品ID，如 BTC-USDT-SWAP，多个产品用半角逗号隔开，最多支持31个产品ID
        返回参数:
        Parameter         	Type    	Description
        instId            	String  	产品id， 如 BTC-USDT-SWAP
        enabled           	Boolean 	true或falsetrue代表设置成功false代表设置失败
        '''
        return self.send_request(*_CopyTradeEndpoints.set_instruments, **to_local(locals()))

    # GET / 交易员历史分润明细
    def get_profit_sharing_details(self, after: str = '', before: str = '', limit: str = '', proxies={},
                                   proxy_host: str = None):
        '''
        交易员获取获取最近三个月的分润明细。
        https://www.okx.com/docs-v5/zh/#order-book-trading-copy-trading-get-profit-sharing-details
        
        限速：5次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        after             	String  	否       	请求此id之前（更旧的数据）的分页内容，传的值为对应接口的profitSharingId
        before            	String  	否       	请求此id之后（更新的数据）的分页内容，传的值为对应接口的profitSharingId
        limit             	String  	否       	分页返回的结果集数量，最大为100，不填默认返回100条
        返回参数:
        Parameter         	Type    	Description
        ccy               	String  	产品ID
        profitSharingAmt  	String  	分润额，没有分润时，默认返回0
        nickName          	String  	跟单人的昵称
        profitSharingId   	String  	分润ID
        ts                	String  	分润时间
        '''
        return self.send_request(*_CopyTradeEndpoints.get_profit_sharing_details, **to_local(locals()))

    # GET / 交易员历史分润汇总
    def get_total_profit_sharing(self, proxies={}, proxy_host: str = None):
        '''
        交易员获取自入驻平台以来，累计获得的总分润金额。
        https://www.okx.com/docs-v5/zh/#order-book-trading-copy-trading-get-total-profit-sharing
        
        限速：5次/2s
        限速规则：UserID
    
        
        返回参数:
        Parameter         	Type    	Description
        ccy               	String  	分润币种
        totalProfitSharingAmt	String  	历史分润汇总
        '''
        return self.send_request(*_CopyTradeEndpoints.get_total_profit_sharing, **to_local(locals()))

    # GET / 交易员待分润明细
    def get_unrealized_profit_sharing_details(self, proxies={}, proxy_host: str = None):
        '''
        交易员获取预计在下一个周期分到的分润金额明细。<!- 1-2-3 -->当有跟单仓位平仓时，待分润明细会进行更新。
        https://www.okx.com/docs-v5/zh/#order-book-trading-copy-trading-get-unrealized-profit-sharing-details
        
        限速：5次/2s
        限速规则：UserID
    
        
        返回参数:
        Parameter         	Type    	Description
        ccy               	String  	分润币种，e.g.USDT
        unrealizedProfitSharingAmt	String  	待分润额
        nickName          	String  	跟单人昵称
        ts                	String  	数据更新时间
        '''
        return self.send_request(*_CopyTradeEndpoints.get_unrealized_profit_sharing_details, **to_local(locals()))



'''
资金账户
https://www.okx.com/docs-v5/zh/#funding-account-rest-api
'''
from paux.param import to_local
from okx.api._client import Client


class _FundingAccountEndpoints():
    get_assert_currencies = ['/api/v5/asset/currencies', 'GET']  # 获取币种列表
    get_balances = ['/api/v5/asset/balances', 'GET']  # 获取资金账户余额
    get_non_tradable_assets = ['/api/v5/asset/non-tradable-assets', 'GET']  # 获取不可交易资产
    get_asset_valuation = ['/api/v5/asset/asset-valuation', 'GET']  # 获取账户资产估值
    set_transfer = ['/api/v5/asset/transfer', 'POST']  # 资金划转
    get_transfer_state = ['/api/v5/asset/transfer-state', 'GET']  # 获取资金划转状态
    get_bills = ['/api/v5/asset/bills', 'GET']  # 获取资金流水
    get_deposit_lightning = ['/api/v5/asset/deposit-lightning', 'GET']  # 闪电网络充币
    get_deposit_address = ['/api/v5/asset/deposit-address', 'GET']  # 获取充值地址信息
    get_deposit_history = ['/api/v5/asset/deposit-history', 'GET']  # 获取充值记录
    set_withdrawal = ['/api/v5/asset/withdrawal', 'POST']  # 提币
    set_withdrawal_lightning = ['/api/v5/asset/withdrawal-lightning', 'POST']  # 闪电网络提币
    set_cancel_withdrawal = ['/api/v5/asset/cancel-withdrawal', 'POST']  # 撤销提币
    get_withdrawal_history = ['/api/v5/asset/withdrawal-history', 'GET']  # 获取提币记录
    get_deposit_withdraw_status = ['/api/v5/asset/deposit-withdraw-status', 'GET']  # 获取充值/提现的详细状态
    set_convert_dust_assets = ['/api/v5/asset/convert-dust-assets', 'POST']  # 小额资产兑换
    get_exchange_list = ['/api/v5/asset/exchange-list', 'GET']  # 获取交易所列表（公共）
    get_convert_currencies = ['/api/v5/asset/convert/currencies', 'GET']  # 获取闪兑币种列表
    get_currency_pair = ['/api/v5/asset/convert/currency-pair', 'GET']  # 获取闪兑币对信息
    set_estimate_quote = ['/api/v5/asset/convert/estimate-quote', 'POST']  # 闪兑预估询价
    set_trade = ['/api/v5/asset/convert/trade', 'POST']  # 闪兑交易
    get_history = ['/api/v5/asset/convert/history', 'GET']  # 获取闪兑交易历史


class FundingAccount(Client):

    # 获取币种列表
    def get_assert_currencies(self, ccy: str = '', proxies={}, proxy_host: str = None):
        '''
        获取平台所有币种列表。
        https://www.okx.com/docs-v5/zh/#funding-account-rest-api-get-currencies
        
        限速：6 次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        ccy               	String  	否       	币种，如BTC支持多币种查询（不超过20个），币种之间半角逗号分隔
        返回参数:
        Parameter         	Type    	Description
        ccy               	String  	币种名称，如BTC
        name              	String  	币种名称，不显示则无对应名称
        logoLink          	String  	币种Logo链接
        chain             	String  	币种链信息有的币种下有多个链，必须要做区分，如USDT下有USDT-ERC20，USDT-TRC20多个链
        canDep            	Boolean 	当前是否可充值false：不可链上充值true：可以链上充值
        canWd             	Boolean 	当前是否可提币false：不可链上提币true：可以链上提币
        canInternal       	Boolean 	当前是否可内部转账false：不可内部转账true：可以内部转账
        minDep            	String  	币种单笔最小充值量
        minWd             	String  	币种单笔最小提币量
        maxWd             	String  	币种单笔最大提币量
        wdTickSz          	String  	提币精度,表示小数点后的位数。提币手续费精度与提币精度保持一致。内部转账提币精度为小数点后8位。
        wdQuota           	String  	过去24小时内提币额度（包含链上提币和内部转账），单位为USD
        usedWdQuota       	String  	过去24小时内已用提币额度，单位为USD
        minFee            	String  	普通地址最小提币手续费数量适用于链上提币
        maxFee            	String  	普通地址最大提币手续费数量适用于链上提币
        minFeeForCtAddr   	String  	合约地址最小提币手续费数量适用于链上提币
        maxFeeForCtAddr   	String  	合约地址最大提币手续费数量适用于链上提币
        mainNet           	Boolean 	当前链是否为主链
        needTag           	Boolean 	当前链是否需要标签（tag/memo）信息，如EOS该字段为true
        minDepArrivalConfirm	String  	充值到账最小网络确认数。币已到账但不可提。
        minWdUnlockConfirm	String  	提现解锁最小网络确认数
        depQuotaFixed     	String  	充币固定限额，单位为USD没有充币限制则返回""
        usedDepQuotaFixed 	String  	已用充币固定额度，单位为USD没有充币限制则返回""
        depQuoteDailyLayer2	String  	Layer2网络每日充值上限
        '''
        return self.send_request(*_FundingAccountEndpoints.get_assert_currencies, **to_local(locals()))

    # 获取资金账户余额
    def get_balances(self, ccy: str = '', proxies={}, proxy_host: str = None):
        '''
        获取资金账户所有资产列表，查询各币种的余额、冻结和可用等信息。
        https://www.okx.com/docs-v5/zh/#funding-account-rest-api-get-balance
        
        限速：6次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        ccy               	String  	否       	币种，如BTC支持多币种查询（不超过20个），币种之间半角逗号分隔
        返回参数:
        Parameter         	Type    	Description
        ccy               	String  	币种，如BTC
        bal               	String  	余额
        frozenBal         	String  	冻结余额
        availBal          	String  	可用余额
        '''
        return self.send_request(*_FundingAccountEndpoints.get_balances, **to_local(locals()))

    # 获取不可交易资产
    def get_non_tradable_assets(self, ccy: str = '', proxies={}, proxy_host: str = None):
        '''
        GET /api/v5/asset/non-tradable-assets
        https://www.okx.com/docs-v5/zh/#funding-account-rest-api-get-non-tradable-assets
        
        限速：6 次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        ccy               	String  	否       	币种，如BTC支持多币种查询（不超过20个），币种之间半角逗号分隔
        返回参数:
        Parameter         	Type    	Description
        ccy               	String  	币种名称，如CELT
        name              	String  	币种中文名称，不显示则无对应名称
        logoLink          	String  	币种Logo链接
        bal               	String  	可提余额
        canWd             	Boolean 	是否可提false: 不可提true: 可提
        chain             	String  	支持提币的链
        minWd             	String  	币种单笔最小提币量
        wdAll             	Boolean 	该币种资产是否必须一次性全部提取
        fee               	String  	提币固定手续费，单位是USDT。提币手续费精度为小数点后8位。
        ctAddr            	String  	合约地址后6位
        wdTickSz          	String  	提币精度,表示小数点后的位数
        needTag           	Boolean 	提币的链是否需要标签（tag/memo）信息
        '''
        return self.send_request(*_FundingAccountEndpoints.get_non_tradable_assets, **to_local(locals()))

    # 获取账户资产估值
    def get_asset_valuation(self, ccy: str = '', proxies={}, proxy_host: str = None):
        '''
        查看账户资产估值
        https://www.okx.com/docs-v5/zh/#funding-account-rest-api-get-account-asset-valuation
        
        限速：1次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        ccy               	String  	否       	资产估值对应的单位BTC 、USDTUSD 、CNY 、JPY、KRW、RUB、EURVND 、IDR 、INR、PHP、THB、TRYAUD 、SGD 、ARS、SAR、AED、IQD默认为  BTC 为单位的估值
        返回参数:
        Parameter         	Type    	Description
        totalBal          	String  	账户总资产估值
        ts                	String  	数据更新时间，Unix时间戳的毫秒数格式，如 1597026383085
        details           	Object  	各个账户的资产估值
        > funding         	String  	资金账户
        > trading         	String  	交易账户
        > classic         	String  	经典账户 (已废弃)
        > earn            	String  	金融账户
        '''
        return self.send_request(*_FundingAccountEndpoints.get_asset_valuation, **to_local(locals()))

    # 资金划转
    def set_transfer(self, ccy: str, amt: str, from_:str, to: str, type: str = '', subAcct: str = '',
                     loanTrans: bool = '', omitPosRisk: str = '', clientId: str = '', proxies={},
                     proxy_host: str = None):
        '''
        调用时，API Key 需要有交易权限。
        https://www.okx.com/docs-v5/zh/#funding-account-rest-api-funds-transfer
        
        限速：1 次/s
        限速规则：UserID + Currency
    
        请求参数:
        Parameter         	Type    	Required	Description

        type              	String  	否       	划转类型0：账户内划转1：母账户转子账户(仅适用于母账户APIKey)2：子账户转母账户(仅适用于母账户APIKey)3：子账户转母账户(仅适用于子账户APIKey)4：子账户转子账户(仅适用于子账户APIKey，且目标账户需要是同一母账户下的其他子账户。子账户主动转出权限默认是关闭的，权限调整参考设置子账户主动转出权限。)默认是0如果您希望通过母账户API Key控制子账户之间的划转，参考接口子账户间资金划转
        ccy               	String  	是       	划转币种，如USDT
        amt               	String  	是       	划转数量
        from              	String  	是       	转出账户6：资金账户18：交易账户
        to                	String  	是       	转入账户6：资金账户18：交易账户
        subAcct           	String  	可选      	子账户名称当type为1/2/4时，该字段必填
        loanTrans         	Boolean 	否       	是否支持跨币种保证金模式或组合保证金模式下的借币转出true：支持借币转出false：不支持借币转出默认为false
        omitPosRisk       	String  	否       	是否忽略仓位风险默认为false仅适用于组合保证金模式
        clientId          	String  	否       	客户自定义ID字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间。
        返回参数:
        Parameter         	Type    	Description
        transId           	String  	划转 ID
        ccy               	String  	划转币种
        from              	String  	转出账户
        amt               	String  	划转量
        to                	String  	转入账户
        clientId          	String  	客户自定义ID
        '''
        data = to_local(locals())
        data['form'] = data['form_']
        del data['from_']

        return self.send_request(*_FundingAccountEndpoints.set_transfer, **data)

    # 获取资金划转状态
    def get_transfer_state(self, transId: str = '', clientId: str = '', type: str = '', proxies={},
                           proxy_host: str = None):
        '''
        获取最近2个星期内的资金划转状态数据
        https://www.okx.com/docs-v5/zh/#funding-account-rest-api-get-funds-transfer-state
        
        限速：10 次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        transId           	String  	可选      	划转IDtransId和clientId必须传一个，若传两个，以transId为主
        clientId          	String  	可选      	客户自定义ID
        type              	String  	否       	划转类型0：账户内划转1：母账户转子账户(仅适用于母账户APIKey)2：子账户转母账户(仅适用于母账户APIKey)3：子账户转母账户(仅适用于子账户APIKey)4：子账户转子账户(仅适用于子账户APIKey，且目标账户需要是同一母账户下的其他子账户)默认是0
        返回参数:
        Parameter         	Type    	Description
        transId           	String  	划转 ID
        clientId          	String  	客户自定义 ID
        ccy               	String  	划转币种
        amt               	String  	划转量
        type              	String  	划转类型0：账户内划转1：母账户转子账户(仅适用于母账户APIKey)2：子账户转母账户(仅适用于母账户APIKey)3：子账户转母账户(仅适用于子账户APIKey)4：子账户转子账户(仅适用于子账户APIKey，且目标账户需要是同一母账户下的其他子账户)
        from              	String  	转出账户6：资金账户18：交易账户
        to                	String  	转入账户6：资金账户18：交易账户
        subAcct           	String  	子账户名称
        instId            	String  	已废弃
        toInstId          	String  	已废弃
        state             	String  	转账状态success：成功pending：处理中failed：失败
        '''
        return self.send_request(*_FundingAccountEndpoints.get_transfer_state, **to_local(locals()))

    # 获取资金流水
    def get_bills(self, ccy: str = '', type: str = '', clientId: str = '', after: str = '', before: str = '',
                  limit: str = '', proxies={}, proxy_host: str = None):
        '''
        查询资金账户账单流水
        https://www.okx.com/docs-v5/zh/#funding-account-rest-api-asset-bills-details
        
        限速：6 次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        ccy               	String  	否       	币种
        type              	String  	否       	请参考官方文档
        clientId          	String  	否       	转账或提币的客户自定义ID字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间。
        after             	String  	否       	查询在此之前的内容，值为时间戳，Unix 时间戳为毫秒数格式，如1597026383085
        before            	String  	否       	查询在此之后的内容，值为时间戳，Unix 时间戳为毫秒数格式，如1597026383085
        limit             	String  	否       	分页返回的结果集数量，最大为 100，不填默认返回 100 条
        返回参数:
        Parameter         	Type    	Description
        billId            	String  	账单 ID
        ccy               	String  	账户余额币种
        clientId          	String  	转账或提币的客户自定义ID
        balChg            	String  	账户层面的余额变动数量
        bal               	String  	账户层面的余额数量
        type              	String  	账单类型
        ts                	String  	账单创建时间，Unix 时间戳的毫秒数格式，如1597026383085
        '''
        return self.send_request(*_FundingAccountEndpoints.get_bills, **to_local(locals()))

    # 闪电网络充币
    def get_deposit_lightning(self, ccy: str, amt: str, to: str = '', proxies={}, proxy_host: str = None):
        '''
        一个用户在最近24小时内可以生成不超过1万个不同的invoice。
        https://www.okx.com/docs-v5/zh/#funding-account-rest-api-lightning-deposits
        
        限速：2次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        ccy               	String  	是       	币种，仅支持BTC
        amt               	String  	是       	充值数量，推荐在0.000001〜0.1之间
        to                	String  	否       	资金充值到账账户6：资金账户18：交易账户默认是6
        返回参数:
        Parameter         	Type    	Description
        invoice           	String  	invoice 号码
        cTime             	String  	生成invoice时间
        '''
        return self.send_request(*_FundingAccountEndpoints.get_deposit_lightning, **to_local(locals()))

    # 获取充值地址信息
    def get_deposit_address(self, ccy: str, proxies={}, proxy_host: str = None):
        '''
        获取各个币种的充值地址，包括曾使用过的老地址。
        https://www.okx.com/docs-v5/zh/#funding-account-rest-api-get-deposit-address
        
        限速：6次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        ccy               	String  	是       	币种，如BTC
        返回参数:
        Parameter         	Type    	Description
        addr              	String  	充值地址
        tag               	String  	部分币种充值需要标签，若不需要则不返回此字段
        memo              	String  	部分币种充值需要 memo，若不需要则不返回此字段
        pmtId             	String  	部分币种充值需要此字段（payment_id），若不需要则不返回此字段
        addrEx            	Object  	充值地址备注，部分币种充值需要，若不需要则不返回此字段如币种TONCOIN的充值地址备注标签名为comment,则该字段返回：{'comment':'123456'}
        ccy               	String  	币种，如BTC
        chain             	String  	币种链信息有的币种下有多个链，必须要做区分，如USDT下有USDT-ERC20，USDT-TRC20多个链
        to                	String  	转入账户6：资金账户18：交易账户
        selected          	Boolean 	该地址是否为页面选中的地址
        ctAddr            	String  	合约地址后6位
        '''
        return self.send_request(*_FundingAccountEndpoints.get_deposit_address, **to_local(locals()))

    # 获取充值记录
    def get_deposit_history(self, ccy: str = '', depId: str = '', fromWdId: str = '', txId: str = '', type: str = '',
                            state: str = '', after: str = '', before: str = '', limit: object = '', proxies={},
                            proxy_host: str = None):
        '''
        根据币种，充值状态，时间范围获取充值记录，按照时间倒序排列，默认返回 100 条数据。<!- 1-2-3 -->支持Websocket订阅，参考充值信息频道。
        https://www.okx.com/docs-v5/zh/#funding-account-rest-api-get-deposit-history
        
        限速：6次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        ccy               	String  	否       	币种名称，如BTC
        depId             	String  	否       	充值记录 ID
        fromWdId          	String  	否       	内部转账发起者提币申请 ID如果该笔充值来自于内部转账，则该字段展示内部转账发起者的提币申请 ID
        txId              	String  	否       	区块转账哈希记录
        type              	String  	否       	充值方式3：内部转账4：链上充值
        state             	String  	否       	充值状态0：等待确认1：确认到账2：充值成功8：因该币种暂停充值而未到账，恢复充值后自动到账11：命中地址黑名单12：账户或充值被冻结13：子账户充值拦截14：KYC限额
        after             	String  	否       	查询在此之前的内容，值为时间戳，Unix 时间戳为毫秒数格式，如1654041600000
        before            	String  	否       	查询在此之后的内容，值为时间戳，Unix 时间戳为毫秒数格式，如1656633600000
        limit             	string  	否       	返回的结果集数量，默认为100，最大为100，不填默认返回100条
        返回参数:
        Parameter         	Type    	Description
        ccy               	String  	币种名称，如BTC
        chain             	String  	币种链信息有的币种下有多个链，必须要做区分，如USDT下有USDT-ERC20，USDT-TRC20多个链
        amt               	String  	充值数量
        from              	String  	充值账户如果该笔充值来自于内部转账，则该字段展示内部转账发起者的账户信息，可以是手机号、邮箱、账户名称，其他情况返回""
        areaCodeFrom      	String  	如果from为手机号，该字段为该手机号的区号
        to                	String  	到账地址如果该笔充值来自于链上充值，则该字段展示链上地址，其他情况返回""
        txId              	String  	区块转账哈希记录
        ts                	String  	充值记录创建时间，Unix 时间戳的毫秒数格式，如1655251200000
        state             	String  	充值状态0：等待确认1：确认到账2：充值成功8：因该币种暂停充值而未到账，恢复充值后自动到账11：命中地址黑名单12：账户或充值被冻结13：子账户充值拦截14：KYC限额
        depId             	String  	充值记录 ID
        fromWdId          	String  	内部转账发起者提币申请 ID如果该笔充值来自于内部转账，则该字段展示内部转账发起者的提币申请 ID，其他情况返回""
        actualDepBlkConfirm	String  	最新的充币网络确认数
        '''
        return self.send_request(*_FundingAccountEndpoints.get_deposit_history, **to_local(locals()))

    # 提币
    def set_withdrawal(self, ccy: str, amt: str, dest: str, toAddr: str, fee: str, chain: str = '', areaCode: str = '',
                       rcvrInfo: str = '', clientId: str = '', proxies={}, proxy_host: str = None):
        '''
        用户提币。普通子账户不支持提币。
        https://www.okx.com/docs-v5/zh/#funding-account-rest-api-withdrawal
        
        限速：6次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        ccy               	String  	是       	币种，如USDT
        amt               	String  	是       	提币数量该数量不包含手续费
        dest              	String  	是       	提币方式3：内部转账4：链上提币
        toAddr            	String  	是       	如果选择链上提币，toAddr必须是认证过的数字货币地址。某些数字货币地址格式为地址:标签，如ARDOR-7JF3-8F2E-QUWZ-CAN7F:123456如果选择内部转账，toAddr必须是接收方地址，可以是邮箱、手机或者账户名（只有子账户才有账户名）。
        fee               	String  	是       	提币到数字货币地址所需网络手续费可以通过接口获取币种列表获取内部转账无需手续费
        chain             	String  	可选      	币种链信息如USDT下有USDT-ERC20，USDT-TRC20多个链如果没有不填此参数，则默认为主链对于无效资产提币，不填此参数，则默认为唯一的提币链适用于链上提币，链信息可以通过接口获取币种列表获取
        areaCode          	String  	可选      	手机区号，如86当toAddr为手机号时，该参数必填适用于内部转账
        rcvrInfo          	String  	可选      	接收方信息特定 国家/地区 认证用户做链上提币需要提供此信息
        > walletType      	String  	是       	钱包类型exchange：提币到交易所钱包private：提币到私人钱包如果提币到交易所钱包，必须提供exchId,rcvrFirstName,rcvrLastName提币到私人钱包，则不需要提供其他信息
        > exchId          	String  	可选      	交易所 ID可以通过获取交易所列表（公共）接口查询支持的交易所如果交易所不在支持的交易所列表中，该字段填0
        > rcvrFirstName   	String  	可选      	接收方名字，如Bruce
        > rcvrLastName    	String  	可选      	接收方姓氏，如Wayne
        clientId          	String  	否       	客户自定义ID字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间。
        返回参数:
        Parameter         	Type    	Description
        ccy               	String  	提币币种
        chain             	String  	币种链信息有的币种下有多个链，必须要做区分，如USDT下有USDT-ERC20，USDT-TRC20多个链
        amt               	String  	提币数量
        wdId              	String  	提币申请ID
        clientId          	String  	客户自定义ID
        '''
        return self.send_request(*_FundingAccountEndpoints.set_withdrawal, **to_local(locals()))

    # 闪电网络提币
    def set_withdrawal_lightning(self, ccy: str, invoice: str, memo: str = '', rcvrInfo: str = '', proxies={},
                                 proxy_host: str = None):
        '''
        单笔提币金额最大值为"0.1BTC"，最小值暂定为"0.000001BTC"，最近24小时内最大提币数量为"1BTC"。子账户不支持提币。
        https://www.okx.com/docs-v5/zh/#funding-account-rest-api-lightning-withdrawals
        
        限速：2次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        ccy               	String  	是       	币种，如BTC
        invoice           	String  	是       	invoice 号码
        memo              	String  	否       	闪电网络提币的备注
        rcvrInfo          	String  	可选      	接收方信息特定 国家/地区 认证用户做链上提币需要提供此信息
        > walletType      	String  	是       	钱包类型exchange：提币到交易所钱包private：提币到私人钱包如果提币到交易所钱包，必须提供exchId,rcvrFirstName,rcvrLastName提币到私人钱包，则不需要提供其他信息
        > exchId          	String  	可选      	交易所 ID可以通过获取交易所列表（公共）接口查询支持的交易所如果交易所不在支持的交易所列表中，该字段填0
        > rcvrFirstName   	String  	可选      	接收方名字，如Bruce
        > rcvrLastName    	String  	可选      	接收方姓氏，如Wayne
        返回参数:
        Parameter         	Type    	Description
        wdId              	String  	提币申请 ID
        cTime             	String  	提币申请 ID生成时间
        '''
        return self.send_request(*_FundingAccountEndpoints.set_withdrawal_lightning, **to_local(locals()))

    # 撤销提币
    def set_cancel_withdrawal(self, wdId: str, proxies={}, proxy_host: str = None):
        '''
        可以撤销普通提币，但不支持撤销闪电网络上的提币。
        https://www.okx.com/docs-v5/zh/#funding-account-rest-api-cancel-withdrawal
        
        限速：6次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        wdId              	String  	是       	提币申请ID
        返回参数:
        Parameter         	Type    	Description
        wdId              	String  	提币申请ID
        '''
        return self.send_request(*_FundingAccountEndpoints.set_cancel_withdrawal, **to_local(locals()))

    # 获取提币记录
    def get_withdrawal_history(self, ccy: str = '', wdId: str = '', clientId: str = '', txId: str = '', type: str = '',
                               state: str = '', after: str = '', before: str = '', limit: object = '', proxies={},
                               proxy_host: str = None):
        '''
        根据币种，提币状态，时间范围获取提币记录，按照时间倒序排列，默认返回100条数据。<!- 1-2-3 -->支持Websocket订阅，参考提币信息频道。
        https://www.okx.com/docs-v5/zh/#funding-account-rest-api-get-withdrawal-history
        
        限速：6 次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        ccy               	String  	否       	币种名称，如BTC
        wdId              	String  	否       	提币申请ID
        clientId          	String  	否       	客户自定义ID字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间。
        txId              	String  	否       	区块转账哈希记录
        type              	String  	否       	提币方式3：内部转账4：链上提币
        state             	String  	否       	提币状态-3：撤销中-2：已撤销-1：失败0：等待提币1：提币中2：提币成功7: 审核通过10: 等待划转4,5,6,8,9,12: 等待客服审核
        after             	String  	否       	查询在此之前的内容，值为时间戳，Unix 时间戳为毫秒数格式，如1654041600000
        before            	String  	否       	查询在此之后的内容，值为时间戳，Unix 时间戳为毫秒数格式，如1656633600000
        limit             	string  	否       	返回的结果集数量，默认为100，最大为100，不填默认返回100条
        返回参数:
        Parameter         	Type    	Description
        ccy               	String  	币种
        chain             	String  	币种链信息有的币种下有多个链，必须要做区分，如USDT下有USDT-ERC20，USDT-TRC20多个链
        nonTradableAsset  	Boolean 	是否为不可交易资产true：不可交易资产，false：可交易资产
        amt               	String  	数量
        ts                	String  	提币申请时间，Unix 时间戳的毫秒数格式，如1655251200000
        from              	String  	提币账户可以是邮箱/手机号
        areaCodeFrom      	String  	如果from为手机号，该字段为该手机号的区号
        to                	String  	收币地址
        areaCodeTo        	String  	如果to为手机号，该字段为该手机号的区号
        tag               	String  	部分币种提币需要标签，若不需要则不返回此字段
        pmtId             	String  	部分币种提币需要此字段（payment_id），若不需要则不返回此字段
        memo              	String  	部分币种提币需要此字段，若不需要则不返回此字段
        addrEx            	Object  	提币地址备注，部分币种提币需要，若不需要则不返回此字段。如币种TONCOIN的提币地址备注标签名为comment,则该字段返回：{'comment':'123456'}
        txId              	String  	提币哈希记录内部转账该字段返回""
        fee               	String  	提币手续费数量
        feeCcy            	String  	提币手续费币种，如USDT
        state             	String  	提币状态-3：撤销中-2：已撤销-1：失败0：等待提币1：提币中2：提币成功7: 审核通过10: 等待划转4,5,6,8,9,12: 等待客服审核
        wdId              	String  	提币申请ID
        clientId          	String  	客户自定义ID
        '''
        return self.send_request(*_FundingAccountEndpoints.get_withdrawal_history, **to_local(locals()))

    # 获取充值/提现的详细状态
    def get_deposit_withdraw_status(self, wdId: str = '', txId: str = '', ccy: str = '', to: str = '', chain: str = '',
                                    proxies={}, proxy_host: str = None):
        '''
        获取充值与提现的详细状态信息与预估完成时间。
        https://www.okx.com/docs-v5/zh/#funding-account-rest-api-get-deposit-withdraw-status
        
        限速：1次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        wdId              	String  	可选      	提币申请ID，用于查询资金提现wdId与txId必传其一也仅可传其一
        txId              	String  	可选      	区块转账哈希记录ID，用于查询资金充值wdId与txId必传其一也仅可传其一
        ccy               	String  	可选      	币种，如USDT查询充值时必填，需要与txId一并提供
        to                	String  	可选      	资金充值到账账户地址查询充值时必填，需要与txId一并提供
        chain             	String  	可选      	币种链信息，例如 USDT-ERC20查询充值时必填，需要与txId一并提供
        返回参数:
        Parameter         	Type    	Description
        estCompleteTime   	String  	预估完成时间时区为 UTC+8；格式为 MM/dd/yyyy, h:mm:ss AM/PMestCompleteTime仅为预估完成时间，仅供参考
        state             	String  	充值/提现的现处于的详细阶段提示冒号前面代表阶段，后面代表状态
        txId              	String  	区块转账哈希记录如查询的是提现，则txId返回为空""，此时提现如生成了txId将一并返回
        wdId              	String  	提币申请ID如查询的是充值，则wdId返回为空""
        '''
        return self.send_request(*_FundingAccountEndpoints.get_deposit_withdraw_status, **to_local(locals()))

    # 小额资产兑换
    def set_convert_dust_assets(self, ccy: object, proxies={}, proxy_host: str = None):
        '''
        将资金账户中的小额资产转化为OKB。24小时之内只能兑换5次。
        https://www.okx.com/docs-v5/zh/#funding-account-rest-api-small-assets-convert
        
        限速：1次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        ccy               	Array of strings	是       	需要转换的币种资产，如 ["BTC","USDT"]
        返回参数:
        Parameter         	Type    	Description
        totalCnvAmt       	String  	兑换后总OKB数量
        details           	Array of objects	各币种资产转换详情
        > ccy             	String  	币种资产,如BTC
        > amt             	String  	转换前币种资产数量
        > cnvAmt          	String  	转换后的OKB数量
        > fee             	String  	兑换手续费，单位为OKB
        '''
        return self.send_request(*_FundingAccountEndpoints.set_convert_dust_assets, **to_local(locals()))

    # 获取交易所列表（公共）
    def get_exchange_list(self, proxies={}, proxy_host: str = None):
        '''
        公共接口无须鉴权
        https://www.okx.com/docs-v5/zh/#funding-account-rest-api-get-exchange-list-public
        
        限速：6次/s
        限速规则：IP
    
        
        返回参数:
        Parameter         	Type    	Description
        exchName          	String  	交易所名称，如1xbet
        exchId            	String  	交易所 ID，如did:ethr:0xfeb4f99829a9acdf52979abee87e83addf22a7e1
        '''
        return self.send_request(*_FundingAccountEndpoints.get_exchange_list, **to_local(locals()))

    # 获取闪兑币种列表
    def get_convert_currencies(self, proxies={}, proxy_host: str = None):
        '''
        GET /api/v5/asset/convert/currencies
        https://www.okx.com/docs-v5/zh/#funding-account-rest-api-get-convert-currencies
        
        限速：6次/s
        限速规则：UserID
    
        
        返回参数:
        Parameter         	Type    	Description
        ccy               	String  	币种名称，如BTC
        min               	String  	支持闪兑的最小值(已废弃)
        max               	String  	支持闪兑的最大值(已废弃)
        '''
        return self.send_request(*_FundingAccountEndpoints.get_convert_currencies, **to_local(locals()))

    # 获取闪兑币对信息
    def get_currency_pair(self, fromCcy: str, toCcy: str, proxies={}, proxy_host: str = None):
        '''
        GET /api/v5/asset/convert/currency-pair
        https://www.okx.com/docs-v5/zh/#funding-account-rest-api-get-convert-currency-pair
        
        限速：6次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        fromCcy           	String  	是       	消耗币种，如USDT
        toCcy             	String  	是       	获取币种，如BTC
        返回参数:
        Parameter         	Type    	Description
        instId            	String  	币对，如BTC-USDT
        baseCcy           	String  	交易货币币种，如BTC-USDT中的BTC
        baseCcyMax        	String  	交易货币支持闪兑的最大值
        baseCcyMin        	String  	交易货币支持闪兑的最小值
        quoteCcy          	String  	计价货币币种，如BTC-USDT中的USDT
        quoteCcyMax       	String  	计价货币支持闪兑的最大值
        quoteCcyMin       	String  	计价货币支持闪兑的最小值
        '''
        return self.send_request(*_FundingAccountEndpoints.get_currency_pair, **to_local(locals()))

    # 闪兑预估询价
    def set_estimate_quote(self, baseCcy: str, quoteCcy: str, side: str, rfqSz: str, rfqSzCcy: str, clQReqId: str = '',
                           tag: str = '', proxies={}, proxy_host: str = None):
        '''
        POST /api/v5/asset/convert/estimate-quote
        https://www.okx.com/docs-v5/zh/#funding-account-rest-api-estimate-quote
        
        限速：10次/s限速：1次/5s
        限速规则：UserID限速规则：Instrument
    
        请求参数:
        Parameter         	Type    	Required	Description

        baseCcy           	String  	是       	交易货币币种，如BTC-USDT中的BTC
        quoteCcy          	String  	是       	计价货币币种，如BTC-USDT中的USDT
        side              	String  	是       	交易方向买：buy卖：sell描述的是对于baseCcy的交易方向
        rfqSz             	String  	是       	询价数量
        rfqSzCcy          	String  	是       	询价币种
        clQReqId          	String  	否       	客户端自定义的订单标识字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间。
        tag               	String  	否       	订单标签适用于broker用户
        返回参数:
        Parameter         	Type    	Description
        quoteTime         	String  	生成报价时间，Unix时间戳的毫秒数格式
        ttlMs             	String  	报价有效期，单位为毫秒
        clQReqId          	String  	客户端自定义的订单标识
        quoteId           	String  	报价ID
        baseCcy           	String  	交易货币币种，如 BTC-USDT 中BTC
        quoteCcy          	String  	计价货币币种，如 BTC-USDT 中USDT
        side              	String  	交易方向买：buy卖：sell
        origRfqSz         	String  	原始报价的数量
        rfqSz             	String  	实际报价的数量
        rfqSzCcy          	String  	报价的币种
        cnvtPx            	String  	闪兑价格，单位为计价币
        baseSz            	String  	闪兑交易币数量
        quoteSz           	String  	闪兑计价币数量
        '''
        return self.send_request(*_FundingAccountEndpoints.set_estimate_quote, **to_local(locals()))

    # 闪兑交易
    def set_trade(self, quoteId: str, baseCcy: str, quoteCcy: str, side: str, sz: str, szCcy: str, clTReqId: str = '',
                  tag: str = '', proxies={}, proxy_host: str = None):
        '''
        闪兑交易前需要先询价。
        https://www.okx.com/docs-v5/zh/#funding-account-rest-api-convert-trade
        
        限速：10次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        quoteId           	String  	是       	报价ID
        baseCcy           	String  	是       	交易货币币种，如BTC-USDT中的BTC
        quoteCcy          	String  	是       	计价货币币种，如BTC-USDT中的USDT
        side              	String  	是       	交易方向buy：买sell：卖描述的是对于baseCcy的交易方向
        sz                	String  	是       	用户报价数量报价数量应不大于预估询价中的询价数量
        szCcy             	String  	是       	用户报价币种
        clTReqId          	String  	否       	用户自定义的订单标识字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间。
        tag               	String  	否       	订单标签适用于broker用户
        返回参数:
        Parameter         	Type    	Description
        tradeId           	String  	成交ID
        quoteId           	String  	报价ID
        clTReqId          	String  	用户自定义的订单标识
        state             	String  	状态fullyFilled：交易成功rejected：交易失败
        instId            	String  	币对，如BTC-USDT
        baseCcy           	String  	交易货币币种，如BTC-USDT中BTC
        quoteCcy          	String  	计价货币币种，如BTC-USDT中USDT
        side              	String  	交易方向买：buy卖：sell
        fillPx            	String  	成交价格，单位为计价币
        fillBaseSz        	String  	成交的交易币数量
        fillQuoteSz       	String  	成交的计价币数量
        ts                	String  	闪兑交易时间，值为时间戳，Unix时间戳为毫秒数格式，如1597026383085
        '''
        return self.send_request(*_FundingAccountEndpoints.set_trade, **to_local(locals()))

    # 获取闪兑交易历史
    def get_history(self, after: str = '', before: str = '', limit: str = '', tag: str = '', proxies={},
                    proxy_host: str = None):
        '''
        GET /api/v5/asset/convert/history
        https://www.okx.com/docs-v5/zh/#funding-account-rest-api-get-convert-history
        
        限速：6次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        after             	String  	否       	查询在此之前的内容，值为时间戳，Unix时间戳为毫秒数格式，如1597026383085
        before            	String  	否       	查询在此之后的内容，值为时间戳，Unix时间戳为毫秒数格式，如1597026383085
        limit             	String  	否       	返回的结果集数量，默认为100，最大为100
        tag               	String  	否       	订单标签适用于broker用户如果闪兑交易带上了tag,查询时必须也带上此参数
        返回参数:
        Parameter         	Type    	Description
        tradeId           	String  	成交ID
        state             	String  	fullyFilled：交易成功rejected：交易失败
        instId            	String  	币对，如BTC-USDT
        baseCcy           	String  	交易货币币种，如BTC-USDT中的BTC
        quoteCcy          	String  	计价货币币种，如BTC-USDT中的USDT
        side              	String  	交易方向买：buy卖：sell
        fillPx            	String  	成交价格，单位为计价币
        fillBaseSz        	String  	成交的交易币数量
        fillQuoteSz       	String  	成交的计价币数量
        ts                	String  	闪兑交易时间，值为时间戳，Unix时间戳为毫秒数格式，如1597026383085
        '''
        return self.send_request(*_FundingAccountEndpoints.get_history, **to_local(locals()))



import traceback
from copy import deepcopy
from pprint import pprint


def get_short_data(data, length, value_omit='......', key_omit='...', ):
    result_data = data
    if type(data) == list:
        if len(data) > length:
            for i in range(length):
                data[i] = get_short_data(data[i], length)
            result_data = data[0:length] + [value_omit, ]
    elif type(data) == dict:
        if len(data) > length:
            result_data = {}
            for i, (sub_key, sub_value) in enumerate(data.items()):
                if i >= length:
                    break
                sub_value = get_short_data(sub_value, length)
                result_data[sub_key] = sub_value
            result_data[key_omit] = value_omit
    return result_data


def eprint(result, length=5, data_length=30, value_omit='......', key_omit='...', width=120):
    result_copy = deepcopy(result)
    if 'data' in result_copy.keys():
        if type(result_copy['data']) == list:
            short_datas = []
            for data in result['data'][0:data_length]:
                short_data = get_short_data(data, length)
                short_datas.append(short_data)
            if len(result['data']) > data_length:
                short_datas.append(value_omit)

        elif type(result_copy['data']) == dict:
            short_datas = {}
            for i, (key, data) in enumerate(result_copy['data'].items()):
                if i > data_length:
                    break
                short_data = get_short_data(data, length)
                short_datas[key] = short_data

            if len(result['data']) > data_length:
                short_datas[key_omit] = value_omit
        else:
            short_datas = result_copy['data']
        result_copy['data'] = short_datas
    pprint(result_copy, sort_dicts=False, width=width)



'''
策略交易
https://www.okx.com/docs-v5/zh/#order-book-trading-algo-trading
'''

from paux.param import to_local
from okx.api._client import Client


class _AlgoTradeEndpoints():
    set_order_algo = ['/api/v5/trade/order-algo', 'POST']  # POST / 策略委托下单
    set_cancel_algos = ['/api/v5/trade/cancel-algos', 'POST']  # POST / 撤销策略委托订单
    set_amend_algos = ['/api/v5/trade/amend-algos', 'POST']  # POST / 修改策略委托订单
    set_cancel_advance_algos = ['/api/v5/trade/cancel-advance-algos', 'POST']  # POST / 撤销高级策略委托订单
    get_order_algo = ['/api/v5/trade/order-algo', 'GET']  # GET / 获取策略委托单信息
    get_orders_algo_pending = ['/api/v5/trade/orders-algo-pending', 'GET']  # GET / 获取未完成策略委托单列表
    get_orders_algo_history = ['/api/v5/trade/orders-algo-history', 'GET']  # GET / 获取历史策略委托单列表


class AlgoTrade(Client):

    # POST / 策略委托下单
    def set_order_algo(self, instId: str, tdMode: str, side: str, ordType: str, ccy: str = '', posSide: str = '',
                       sz: str = '', tag: str = '', tgtCcy: str = '', algoClOrdId: str = '', closeFraction: str = '',
                       proxies={}, proxy_host: str = None):
        '''
        提供单向止盈止损委托、双向止盈止损委托、计划委托、时间加权委托、移动止盈止损委托
        https://www.okx.com/docs-v5/zh/#order-book-trading-algo-trading-post-place-algo-order
        
        限速：20次/2s跟单交易带单合约的限速：1次/2s
        
    
        请求参数:
        Parameter         	Type    	Required	Description

        instId            	String  	是       	产品ID，如BTC-USD-190927-5000-C
        tdMode            	String  	是       	交易模式保证金模式isolated：逐仓，cross：全仓非保证金模式cash：非保证金
        ccy               	String  	否       	保证金币种仅适用于单币种保证金模式下的全仓杠杆订单
        side              	String  	是       	订单方向buy：买sell：卖
        posSide           	String  	可选      	持仓方向在开平仓模式下必填，且仅可选择long或short
        ordType           	String  	是       	订单类型conditional：单向止盈止损oco：双向止盈止损trigger：计划委托move_order_stop：移动止盈止损twap：时间加权委托
        sz                	String  	可选      	委托数量sz和closeFraction必填且只能填其一
        tag               	String  	否       	订单标签字母（区分大小写）与数字的组合，可以是纯字母、纯数字，且长度在1-16位之间
        tgtCcy            	String  	否       	委托数量的类型base_ccy: 交易货币 ；quote_ccy：计价货币仅适用于币币单向止盈止损市价买单默认买为计价货币，卖为交易货币
        algoClOrdId       	String  	否       	客户自定义策略订单ID字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间。
        closeFraction     	String  	可选      	策略委托触发时，平仓的百分比。1 代表100%现在系统只支持全部平仓，唯一接受参数为1对于同一个仓位，仅支持一笔全部平仓的止盈止损挂单仅适用于交割或永续当posSide=net时，reduceOnly必须为true仅适用于止盈止损ordType=conditional或oco仅适用于止盈止损市价订单不支持组合保证金模式sz和closeFraction必填且只能填其一
        返回参数:
        Parameter         	Type    	Description
        algoId            	String  	策略委托单ID
        clOrdId           	String  	客户自定义订单ID（已废弃）
        algoClOrdId       	String  	客户自定义策略订单ID
        sCode             	String  	事件执行结果的code，0代表成功
        sMsg              	String  	事件执行失败时的msg
        tag               	String  	订单标签
        '''
        return self.send_request(*_AlgoTradeEndpoints.set_order_algo, **to_local(locals()))

    # POST / 撤销策略委托订单
    def set_cancel_algos(self, algoId: str, instId: str, proxies={}, proxy_host: str = None):
        '''
        撤销策略委托订单（不包含冰山委托、时间加权、移动止盈止损等高级策略订单），每次最多可以撤销10个策略委托单
        https://www.okx.com/docs-v5/zh/#order-book-trading-algo-trading-post-cancel-algo-order
        
        限速：20次/2s
        
    
        请求参数:
        Parameter         	Type    	Required	Description

        algoId            	String  	是       	策略委托单ID
        instId            	String  	是       	产品ID 如BTC-USDT
        返回参数:
        Parameter         	Type    	Description
        algoId            	String  	订单ID
        sCode             	String  	事件执行结果的code，0代表成功
        sMsg              	String  	事件执行失败时的msg
        '''
        return self.send_request(*_AlgoTradeEndpoints.set_cancel_algos, **to_local(locals()))

    # POST / 修改策略委托订单
    def set_amend_algos(self, instId: str, algoId: str = '', algoClOrdId: str = '', cxlOnFail: bool = '',
                        reqId: str = '', newSz: str = '', newTpTriggerPx: str = '', newTpOrdPx: str = '',
                        newSlTriggerPx: str = '', newSlOrdPx: str = '', newTpTriggerPxType: str = '',
                        newSlTriggerPxType: str = '', proxies={}, proxy_host: str = None):
        '''
        修改策略委托订单（仅支持止盈止损单，不包含计划委托、冰山委托、时间加权、移动止盈止损等订单）<!- 1-2-3 -->只适用于交割和永续合约。
        https://www.okx.com/docs-v5/zh/#order-book-trading-algo-trading-post-amend-algo-order
        
        限速：20次/2s
        限速规则：UserID + Instrument ID
    
        请求参数:
        Parameter         	Type    	Required	Description

        instId            	String  	是       	产品ID
        algoId            	String  	可选      	策略委托单IDalgoId和algoClOrdId必须传一个，若传两个，以algoId为主
        algoClOrdId       	String  	可选      	客户自定义策略订单IDalgoId和algoClOrdId必须传一个，若传两个，以algoId为主
        cxlOnFail         	Boolean 	否       	false：不自动撤单 true：自动撤单 当订单修改失败时，该订单是否需要自动撤销。默认为false
        reqId             	String  	否       	用户自定义修改事件ID，字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间
        newSz             	String  	可选      	修改的新数量
        newTpTriggerPx    	String  	可选      	止盈触发价如果止盈触发价或者委托价为0，那代表删除止盈
        newTpOrdPx        	String  	可选      	止盈委托价委托价格为-1时，执行市价止盈
        newSlTriggerPx    	String  	可选      	止损触发价如果止损触发价或者委托价为0，那代表删除止损
        newSlOrdPx        	String  	可选      	止损委托价委托价格为-1时，执行市价止损
        newTpTriggerPxType	String  	可选      	止盈触发价类型last：最新价格index：指数价格mark：标记价格
        newSlTriggerPxType	String  	可选      	止损触发价类型last：最新价格index：指数价格mark：标记价格
        返回参数:
        Parameter         	Type    	Description
        algoId            	String  	订单ID
        algoClOrdId       	String  	客户自定义策略订单ID
        reqId             	String  	用户自定义修改事件ID，字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间
        sCode             	String  	事件执行结果的code，0代表成功
        sMsg              	String  	事件执行失败时的msg
        '''
        return self.send_request(*_AlgoTradeEndpoints.set_amend_algos, **to_local(locals()))

    # POST / 撤销高级策略委托订单
    def set_cancel_advance_algos(self, algoId: str, instId: str, proxies={}, proxy_host: str = None):
        '''
        撤销冰山委托、时间加权、移动止盈止损委托订单，每次最多可以撤销10个策略委托单
        https://www.okx.com/docs-v5/zh/#order-book-trading-algo-trading-post-cancel-advance-algo-order
        
        限速：20次/2s
        
    
        请求参数:
        Parameter         	Type    	Required	Description

        algoId            	String  	是       	策略委托单ID
        instId            	String  	是       	产品ID 如BTC-USDT
        返回参数:
        Parameter         	Type    	Description
        algoId            	String  	订单ID
        sCode             	String  	事件执行结果的code，0代表成功
        sMsg              	String  	事件执行失败时的msg
        '''
        return self.send_request(*_AlgoTradeEndpoints.set_cancel_advance_algos, **to_local(locals()))

    # GET / 获取策略委托单信息
    def get_order_algo(self, algoId: str = '', algoClOrdId: str = '', proxies={}, proxy_host: str = None):
        '''
        GET /api/v5/trade/order-algo
        https://www.okx.com/docs-v5/zh/#order-book-trading-algo-trading-get-algo-order-details
        
        限速：20次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        algoId            	String  	可选      	策略委托单IDalgoId和algoClOrdId必须传一个，若传两个，以algoId为主
        algoClOrdId       	String  	可选      	客户自定义策略订单ID字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间。
        返回参数:
        Parameter         	Type    	Description
        instType          	String  	产品类型
        instId            	String  	产品ID
        ccy               	String  	保证金币种，仅适用于单币种保证金模式下的全仓杠杆订单
        ordId             	String  	最新一笔订单ID
        ordIdList         	String  	订单ID列表，当止盈止损存在市价拆单时，会有多个。
        algoId            	String  	策略委托单ID
        clOrdId           	String  	客户自定义订单ID
        sz                	String  	委托数量
        closeFraction     	String  	策略委托触发时，平仓的百分比。1 代表100%
        ordType           	String  	订单类型
        side              	String  	订单方向
        posSide           	String  	持仓方向
        tdMode            	String  	交易模式
        tgtCcy            	String  	币币市价单委托数量sz的单位base_ccy: 交易货币 ；quote_ccy：计价货币仅适用于币币市价订单默认买单为quote_ccy，卖单为base_ccy
        state             	String  	订单状态live：待生效pause：暂停生效partially_effective:部分生效effective： 已生效canceled：已撤销order_failed：委托失败partially_failed：部分委托失败
        lever             	String  	杠杆倍数，0.01到125之间的数值，仅适用于币币杠杆/交割/永续
        tpTriggerPx       	String  	止盈触发价
        tpTriggerPxType   	String  	止盈触发价类型last：最新价格index：指数价格mark：标记价格
        tpOrdPx           	String  	止盈委托价
        slTriggerPx       	String  	止损触发价
        slTriggerPxType   	String  	止损触发价类型last：最新价格index：指数价格mark：标记价格
        slOrdPx           	String  	止损委托价
        triggerPx         	String  	计划委托触发价格
        triggerPxType     	String  	计划委托触发价格类型last：最新价格index：指数价格mark：标记价格
        ordPx             	String  	订单委托价格
        actualSz          	String  	实际委托量
        actualPx          	String  	实际委托价
        actualSide        	String  	实际触发方向tp：止盈sl： 止损
        triggerTime       	String  	策略委托触发时间，Unix时间戳的毫秒数格式，如1597026383085
        pxVar             	String  	价格比例仅适用于冰山委托和时间加权委托
        pxSpread          	String  	价距仅适用于冰山委托和时间加权委托
        szLimit           	String  	单笔数量仅适用于冰山委托和时间加权委托
        pxLimit           	String  	挂单限制价仅适用于冰山委托和时间加权委托
        tag               	String  	订单标签
        timeInterval      	String  	下单间隔仅适用于时间加权委托
        callbackRatio     	String  	回调幅度的比例仅适用于移动止盈止损
        callbackSpread    	String  	回调幅度的价距仅适用于移动止盈止损
        activePx          	String  	移动止盈止损激活价格仅适用于移动止盈止损
        moveTriggerPx     	String  	移动止盈止损触发价格仅适用于移动止盈止损
        reduceOnly        	String  	是否只减仓，true或false
        quickMgnType      	String  	一键借币类型，仅适用于杠杆逐仓的一键借币模式manual：手动，auto_borrow： 自动借币，auto_repay： 自动还币
        last              	String  	下单时的最新成交价
        failCode          	String  	代表策略触发失败的原因，已撤销和已生效时为""，委托失败时有值，如 51008；仅适用于单向止盈止损委托、双向止盈止损委托、移动止盈止损委托、计划委托。
        algoClOrdId       	String  	客户自定义策略订单ID
        attachAlgoOrds    	Array of object	附带止盈止损信息适用于单币种保证金模式/跨币种保证金/模式组合保证金模式
        > attachAlgoClOrdId	String  	下单附带止盈止损时，客户自定义的策略订单ID，字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间。订单完全成交，下止盈止损委托单时，该值会传给algoClOrdId。
        > tpTriggerPx     	String  	止盈触发价，如果填写此参数，必须填写止盈委托价
        > tpTriggerPxType 	String  	止盈触发价类型last：最新价格index：指数价格mark：标记价格默认为last
        > tpOrdPx         	String  	止盈委托价，如果填写此参数，必须填写止盈触发价委托价格为-1时，执行市价止盈
        > slTriggerPx     	String  	止损触发价，如果填写此参数，必须填写止损委托价
        > slTriggerPxType 	String  	止损触发价类型last：最新价格index：指数价格mark：标记价格默认为last
        > slOrdPx         	String  	止损委托价，如果填写此参数，必须填写止损触发价委托价格为-1时，执行市价止损
        cTime             	String  	订单创建时间，Unix时间戳的毫秒数格式，如1597026383085
        '''
        return self.send_request(*_AlgoTradeEndpoints.get_order_algo, **to_local(locals()))

    # GET / 获取未完成策略委托单列表
    def get_orders_algo_pending(self, ordType: str, algoId: str = '', instType: str = '', instId: str = '',
                                after: str = '', before: str = '', limit: str = '', algoClOrdId: str = '', proxies={},
                                proxy_host: str = None):
        '''
        获取当前账户下未触发的策略委托单列表
        https://www.okx.com/docs-v5/zh/#order-book-trading-algo-trading-get-algo-order-list
        
        限速：20次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        algoId            	String  	否       	策略委托单ID
        instType          	String  	否       	产品类型SPOT：币币SWAP：永续合约FUTURES：交割合约MARGIN：杠杆
        instId            	String  	否       	产品ID，BTC-USD-190927
        ordType           	String  	是       	订单类型conditional：单向止盈止损oco：双向止盈止损trigger：计划委托move_order_stop：移动止盈止损iceberg：冰山委托twap：时间加权委托
        after             	String  	否       	请求此ID之前（更旧的数据）的分页内容，传的值为对应接口的algoId
        before            	String  	否       	请求此ID之后（更新的数据）的分页内容，传的值为对应接口的algoId
        limit             	String  	否       	返回结果的数量，最大为100，默认100条
        algoClOrdId       	String  	否       	客户自定义策略订单ID字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间。
        返回参数:
        Parameter         	Type    	Description
        instType          	String  	产品类型
        instId            	String  	产品ID
        ccy               	String  	保证金币种，仅适用于单币种保证金模式下的全仓杠杆订单
        ordId             	String  	最新一笔订单ID
        ordIdList         	String  	订单ID列表，当止盈止损存在市价拆单时，会有多个。
        algoId            	String  	策略委托单ID
        clOrdId           	String  	客户自定义订单ID
        sz                	String  	委托数量
        closeFraction     	String  	策略委托触发时，平仓的百分比。1 代表100%
        ordType           	String  	订单类型
        side              	String  	订单方向
        posSide           	String  	持仓方向
        tdMode            	String  	交易模式
        tgtCcy            	String  	币币市价单委托数量sz的单位base_ccy: 交易货币 ；quote_ccy：计价货币仅适用于币币市价订单默认买单为quote_ccy，卖单为base_ccy
        state             	String  	订单状态 ，live：待生效pause：暂停生效
        lever             	String  	杠杆倍数，0.01到125之间的数值，仅适用于币币杠杆/交割/永续
        tpTriggerPx       	String  	止盈触发价
        tpTriggerPxType   	String  	止盈触发价类型last：最新价格index：指数价格mark：标记价格
        tpOrdPx           	String  	止盈委托价
        slTriggerPx       	String  	止损触发价
        slTriggerPxType   	String  	止损触发价类型last：最新价格index：指数价格mark：标记价格
        slOrdPx           	String  	止损委托价
        triggerPx         	String  	计划委托触发价格
        triggerPxType     	String  	计划委托触发价类型last：最新价格index：指数价格mark：标记价格
        ordPx             	String  	计划委托委托价格
        actualSz          	String  	实际委托量
        actualPx          	String  	实际委托价
        actualSide        	String  	实际触发方向，tp：止盈sl： 止损
        triggerTime       	String  	策略委托触发时间，Unix时间戳的毫秒数格式，如1597026383085
        pxVar             	String  	价格比例仅适用于冰山委托和时间加权委托
        pxSpread          	String  	价距仅适用于冰山委托和时间加权委托
        szLimit           	String  	单笔数量仅适用于冰山委托和时间加权委托
        tag               	String  	订单标签
        pxLimit           	String  	挂单限制价仅适用于冰山委托和时间加权委托
        timeInterval      	String  	下单间隔仅适用于时间加权委托
        callbackRatio     	String  	回调幅度的比例仅适用于移动止盈止损
        callbackSpread    	String  	回调幅度的价距仅适用于移动止盈止损
        activePx          	String  	移动止盈止损激活价格仅适用于移动止盈止损
        moveTriggerPx     	String  	移动止盈止损触发价格仅适用于移动止盈止损
        reduceOnly        	String  	是否只减仓，true或false
        quickMgnType      	String  	一键借币类型，仅适用于杠杆逐仓的一键借币模式manual：手动，auto_borrow： 自动借币，auto_repay： 自动还币
        last              	String  	下单时的最新成交价
        failCode          	String  	代表策略触发失败的原因，委托失败时有值，如 51008，对于该接口一直为""。
        algoClOrdId       	String  	客户自定义策略订单ID
        attachAlgoOrds    	Array of object	附带止盈止损信息适用于单币种保证金模式/跨币种保证金/模式组合保证金模式
        > attachAlgoClOrdId	String  	下单附带止盈止损时，客户自定义的策略订单ID，字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间。订单完全成交，下止盈止损委托单时，该值会传给algoClOrdId。
        > tpTriggerPx     	String  	止盈触发价，如果填写此参数，必须填写止盈委托价
        > tpTriggerPxType 	String  	止盈触发价类型last：最新价格index：指数价格mark：标记价格默认为last
        > tpOrdPx         	String  	止盈委托价，如果填写此参数，必须填写止盈触发价委托价格为-1时，执行市价止盈
        > slTriggerPx     	String  	止损触发价，如果填写此参数，必须填写止损委托价
        > slTriggerPxType 	String  	止损触发价类型last：最新价格index：指数价格mark：标记价格默认为last
        > slOrdPx         	String  	止损委托价，如果填写此参数，必须填写止损触发价委托价格为-1时，执行市价止损
        cTime             	String  	订单创建时间，   Unix时间戳的毫秒数格式，如1597026383085
        '''
        return self.send_request(*_AlgoTradeEndpoints.get_orders_algo_pending, **to_local(locals()))

    # GET / 获取历史策略委托单列表
    def get_orders_algo_history(self, ordType: str, state: str = '', algoId: str = '', instType: str = '',
                                instId: str = '', after: str = '', before: str = '', limit: str = '', proxies={},
                                proxy_host: str = None):
        '''
        获取最近3个月当前账户下所有策略委托单列表
        https://www.okx.com/docs-v5/zh/#order-book-trading-algo-trading-get-algo-order-history
        
        限速：20次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        ordType           	String  	是       	订单类型conditional：单向止盈止损oco：双向止盈止损trigger：计划委托move_order_stop：移动止盈止损iceberg：冰山委托twap：时间加权委托
        state             	String  	可选      	订单状态effective：已生效canceled：已经撤销order_failed：委托失败state和algoId必填且只能填其一
        algoId            	String  	可选      	策略委托单ID
        instType          	String  	否       	产品类型SPOT：币币SWAP：永续合约FUTURES：交割合约MARGIN：杠杆
        instId            	String  	否       	产品ID，BTC-USD-190927
        after             	String  	否       	请求此ID之前（更旧的数据）的分页内容，传的值为对应接口的algoId
        before            	String  	否       	请求此ID之后（更新的数据）的分页内容，传的值为对应接口的algoId
        limit             	String  	否       	返回结果的数量，最大为100，默认100条
        返回参数:
        Parameter         	Type    	Description
        instType          	String  	产品类型
        instId            	String  	产品ID
        ccy               	String  	保证金币种，仅适用于单币种保证金模式下的全仓杠杆订单
        ordId             	String  	最新一笔订单ID
        ordIdList         	String  	订单ID列表，当止盈止损存在市价拆单时，会有多个。
        algoId            	String  	策略委托单ID
        clOrdId           	String  	客户自定义订单ID
        sz                	String  	委托数量
        closeFraction     	String  	策略委托触发时，平仓的百分比。1 代表100%
        ordType           	String  	订单类型
        side              	String  	订单方向
        posSide           	String  	持仓方向
        tdMode            	String  	交易模式
        tgtCcy            	String  	币币市价单委托数量sz的单位base_ccy: 交易货币 ；quote_ccy：计价货币仅适用于币币市价订单默认买单为quote_ccy，卖单为base_ccy
        state             	String  	订单状态effective： 已生效canceled：已撤销order_failed：委托失败partially_failed：部分委托失败
        lever             	String  	杠杆倍数，0.01到125之间的数值，仅适用于币币杠杆/交割/永续
        tpTriggerPx       	String  	止盈触发价
        tpTriggerPxType   	String  	止盈触发价类型last：最新价格index：指数价格mark：标记价格
        tpOrdPx           	String  	止盈委托价
        slTriggerPx       	String  	止损触发价
        slTriggerPxType   	String  	止损触发价类型last：最新价格index：指数价格mark：标记价格
        slOrdPx           	String  	止损委托价
        triggerPx         	String  	计划委托触发价格
        triggerPxType     	String  	计划委托触发价格
        ordPx             	String  	计划委托委托价格类型last：最新价格index：指数价格mark：标记价格
        actualSz          	String  	实际委托量
        actualPx          	String  	实际委托价
        actualSide        	String  	实际触发方向tp：止盈sl： 止损
        triggerTime       	String  	策略委托触发时间，Unix时间戳的毫秒数格式，如1597026383085
        pxVar             	String  	价格比例仅适用于冰山委托和时间加权委托
        pxSpread          	String  	价距仅适用于冰山委托和时间加权委托
        szLimit           	String  	单笔数量仅适用于冰山委托和时间加权委托
        pxLimit           	String  	挂单限制价仅适用于冰山委托和时间加权委托
        tag               	String  	订单标签
        timeInterval      	String  	下单间隔仅适用于时间加权委托
        callbackRatio     	String  	回调幅度的比例仅适用于移动止盈止损
        callbackSpread    	String  	回调幅度的价距仅适用于移动止盈止损
        activePx          	String  	移动止盈止损激活价格仅适用于移动止盈止损
        moveTriggerPx     	String  	移动止盈止损触发价格仅适用于移动止盈止损
        reduceOnly        	String  	是否只减仓，true或false
        quickMgnType      	String  	一键借币类型，仅适用于杠杆逐仓的一键借币模式manual：手动，auto_borrow： 自动借币，auto_repay： 自动还币
        last              	String  	下单时的最新成交价
        failCode          	String  	代表策略触发失败的原因，已撤销和已生效时为""，委托失败时有值，如 51008；仅适用于单向止盈止损委托、双向止盈止损委托、移动止盈止损委托、计划委托。
        algoClOrdId       	String  	客户自定义策略订单ID
        attachAlgoOrds    	Array of object	附带止盈止损信息适用于单币种保证金模式/跨币种保证金/模式组合保证金模式
        > attachAlgoClOrdId	String  	下单附带止盈止损时，客户自定义的策略订单ID，字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间。订单完全成交，下止盈止损委托单时，该值会传给algoClOrdId。
        > tpTriggerPx     	String  	止盈触发价，如果填写此参数，必须填写止盈委托价
        > tpTriggerPxType 	String  	止盈触发价类型last：最新价格index：指数价格mark：标记价格默认为last
        > tpOrdPx         	String  	止盈委托价，如果填写此参数，必须填写止盈触发价委托价格为-1时，执行市价止盈
        > slTriggerPx     	String  	止损触发价，如果填写此参数，必须填写止损委托价
        > slTriggerPxType 	String  	止损触发价类型last：最新价格index：指数价格mark：标记价格默认为last
        > slOrdPx         	String  	止损委托价，如果填写此参数，必须填写止损触发价委托价格为-1时，执行市价止损
        cTime             	String  	订单创建时间，Unix时间戳的毫秒数格式，如1597026383085
        '''
        return self.send_request(*_AlgoTradeEndpoints.get_orders_algo_history, **to_local(locals()))



'''
余币宝
https://www.okx.com/docs-v5/zh/#financial-product-savings
'''
from paux.param import to_local
from okx.api._client import Client


class _SavingsEndpoints():
    get_balance = ['/api/v5/finance/savings/balance', 'GET']  # GET / 获取余币宝余额
    set_purchase_redempt = ['/api/v5/finance/savings/purchase-redempt', 'POST']  # POST / 余币宝申购/赎回
    set_lending_rate = ['/api/v5/finance/savings/set-lending-rate', 'POST']  # POST / 设置余币宝借贷利率
    get_lending_history = ['/api/v5/finance/savings/lending-history', 'GET']  # GET / 获取余币宝出借明细
    get_lending_rate_summary = ['/api/v5/finance/savings/lending-rate-summary', 'GET']  # GET / 获取市场借贷信息（公共）
    get_lending_rate_history = ['/api/v5/finance/savings/lending-rate-history', 'GET']  # GET / 获取市场借贷历史（公共）


class Savings(Client):

    # GET / 获取余币宝余额
    def get_balance(self, ccy: str = '', proxies={}, proxy_host: str = None):
        '''
        GET /api/v5/finance/savings/balance
        https://www.okx.com/docs-v5/zh/#financial-product-savings-get-saving-balance
        
        限速：6次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        ccy               	String  	否       	币种，如BTC
        返回参数:
        Parameter         	Type    	Description
        ccy               	String  	币种，如BTC
        amt               	String  	币种数量
        earnings          	String  	币种持仓收益
        rate              	String  	最新出借利率
        loanAmt           	String  	已出借数量
        pendingAmt        	String  	未出借数量
        redemptAmt        	String  	赎回中的数量（已废弃）
        '''
        return self.send_request(*_SavingsEndpoints.get_balance, **to_local(locals()))

    # POST / 余币宝申购/赎回
    def set_purchase_redempt(self, ccy: str, amt: str, side: str, rate: str, proxies={}, proxy_host: str = None):
        '''
        仅资金账户中的资产支持余币宝申购。
        https://www.okx.com/docs-v5/zh/#financial-product-savings-post-savings-purchase-redemption
        
        限速：6次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        ccy               	String  	是       	币种名称，如BTC
        amt               	String  	是       	申购/赎回 数量
        side              	String  	是       	操作类型purchase：申购redempt：赎回
        rate              	String  	是       	申购年利率仅适用于申购，新申购的利率会覆盖上次申购的利率参数取值范围在1%到365%之间
        返回参数:
        Parameter         	Type    	Description
        ccy               	String  	币种名称
        amt               	String  	申购/赎回 数量
        side              	String  	操作类型
        rate              	String  	申购年利率
        '''
        return self.send_request(*_SavingsEndpoints.set_purchase_redempt, **to_local(locals()))

    # POST / 设置余币宝借贷利率
    def set_lending_rate(self, ccy: str, rate: str, proxies={}, proxy_host: str = None):
        '''
        POST /api/v5/finance/savings/set-lending-rate
        https://www.okx.com/docs-v5/zh/#financial-product-savings-post-set-lending-rate
        
        限速：6次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        ccy               	String  	是       	币种名称，如BTC
        rate              	String  	是       	贷出年利率参数取值范围在1%到365%之间
        返回参数:
        Parameter         	Type    	Description
        ccy               	String  	币种名称，如BTC
        rate              	String  	贷出年利率
        '''
        return self.send_request(*_SavingsEndpoints.set_lending_rate, **to_local(locals()))

    # GET / 获取余币宝出借明细
    def get_lending_history(self, ccy: str = '', after: str = '', before: str = '', limit: str = '', proxies={},
                            proxy_host: str = None):
        '''
        GET /api/v5/finance/savings/lending-history
        https://www.okx.com/docs-v5/zh/#financial-product-savings-get-lending-history
        
        限速：6次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        ccy               	String  	否       	币种，如BTC
        after             	String  	否       	查询在此之前的内容，值为时间戳，Unix 时间戳为毫秒数格式，如1597026383085
        before            	String  	否       	查询在此之后的内容，值为时间戳，Unix 时间戳为毫秒数格式，如1597026383085
        limit             	String  	否       	分页返回的结果集数量，最大为 100，不填默认返回 100 条
        返回参数:
        Parameter         	Type    	Description
        ccy               	String  	币种，如BTC
        amt               	String  	出借数量
        earnings          	String  	已赚取利息
        rate              	String  	出借年利率
        ts                	String  	出借时间，Unix时间戳的毫秒数格式，如1597026383085
        '''
        return self.send_request(*_SavingsEndpoints.get_lending_history, **to_local(locals()))

    # GET / 获取市场借贷信息（公共）
    def get_lending_rate_summary(self, ccy: str = '', proxies={}, proxy_host: str = None):
        '''
        公共接口无须鉴权
        https://www.okx.com/docs-v5/zh/#financial-product-savings-get-public-borrow-info-public
        
        限速：6次/s
        限速规则：IP
    
        请求参数:
        Parameter         	Type    	Required	Description

        ccy               	String  	否       	币种，如BTC
        返回参数:
        Parameter         	Type    	Description
        ccy               	String  	币种，如BTC
        avgAmt            	String  	过去24小时平均借贷量
        avgAmtUsd         	String  	过去24小时平均借贷美元价值
        avgRate           	String  	过去24小时平均借出利率
        preRate           	String  	上一次借贷年利率
        estRate           	String  	下一次预估借贷年利率
        '''
        return self.send_request(*_SavingsEndpoints.get_lending_rate_summary, **to_local(locals()))

    # GET / 获取市场借贷历史（公共）
    def get_lending_rate_history(self, ccy: str = '', after: str = '', before: str = '', limit: str = '', proxies={},
                                 proxy_host: str = None):
        '''
        公共接口无须鉴权<!- 1-2-3 -->返回2021年12月14日后的记录
        https://www.okx.com/docs-v5/zh/#financial-product-savings-get-public-borrow-history-public
        
        限速：6次/s
        限速规则：IP
    
        请求参数:
        Parameter         	Type    	Required	Description

        ccy               	String  	否       	币种，如BTC
        after             	String  	否       	查询在此之前的内容，值为时间戳，Unix 时间戳为毫秒数格式，如1597026383085
        before            	String  	否       	查询在此之后的内容，值为时间戳，Unix 时间戳为毫秒数格式，如1597026383085
        limit             	String  	否       	分页返回的结果集数量，最大为100，不填默认返回100条如果不指定ccy,会返回同一个ts下的全部数据，不受limit限制
        返回参数:
        Parameter         	Type    	Description
        ccy               	String  	币种，如BTC
        amt               	String  	市场总出借数量
        rate              	String  	出借年利率
        ts                	String  	时间，Unix时间戳的毫秒数格式，如1597026383085
        '''
        return self.send_request(*_SavingsEndpoints.get_lending_rate_history, **to_local(locals()))



from okx.app.account import AccountSPOT
from okx.app.market import MarketSPOT
from okx.api import Trade as TradeAPI


class TradeBase():
    def __init__(
            self,
            key: str,
            secret: str,
            passphrase: str,
            timezone: str = 'Asia/Shanghai',
            account=None,
            market=None,
            proxies={},
            proxy_host: str = None,
    ):
        # SWAP账户
        if not account:
            self._account = AccountSPOT(key=key, secret=secret, passphrase=passphrase, proxies=proxies,
                                        proxy_host=proxy_host)
        else:
            self._account = account

        # SWAP行情
        if not market:
            self._market = MarketSPOT(key=key, secret=secret, passphrase=passphrase, timezone=timezone, proxies=proxies,
                                      proxy_host=proxy_host)
        else:
            self._market = market

        # 时区
        self.timezone = timezone
        # TRADE API
        FLAG = '0'
        self.api = TradeAPI(key=key, secret=secret, passphrase=passphrase, flag=FLAG, proxies=proxies,
                            proxy_host=proxy_host)
